/** Classe de base pour les exercices des logiciels AbulEdu
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/LeTerrierExercice
  * @author 2012-2013 Éric Seigne <eric.seigne@ryxeo.com>
  * @author 2012-2013 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * @author 2012-2013 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @author 2013 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "exerciceqcm.h"

ExerciceQCM::ExerciceQCM(QWidget *parent, QString repertoireBase) :
    AbulEduCommonStatesV1(parent),
    m_parent(parent),
    m_repertoireBase(repertoireBase),
    m_className("ExerciceQCM"),
    m_questionPosee(""),
    m_reponseDonnee(""),
    m_score(-1),
    m_nbPrintedQuestions(0),
    m_reponseAttendue(""),
    m_tempsReponse("")
{
    m_localDebug = true;
    m_resultats.clear();
    setAbeExerciceEvaluation(abe::evalY);
    abeStateMachineSetNbClicSurVerifier(0);
    
    
    /* Instanciation du fichier de configuration */
    if(!QDir(repertoireBase).exists())
    {
        AbulEduMessageBoxV1(trUtf8("Échec"), trUtf8("Impossible d'ouvrir le fichier spécifié !"));
        slotQuitterAccueil();
    }
    else
    {
        m_config = new QSettings(repertoireBase + "/conf/module.conf",QSettings::IniFormat);
    }
    
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    
    setAbeExerciceName(trUtf8("Quizz"));
    
    m_levels.insert(1, trUtf8("Très facile"));
    m_levels.insert(2, trUtf8("Facile"));
    m_levels.insert(3, trUtf8("Moyen"));
    m_levels.insert(4, trUtf8("Difficile"));
    m_levels.insert(5, trUtf8("Très difficile"));
    
    /* Création de l'aire de jeu et de sa scène */
    gv_AireDeJeu = new QGraphicsView();
    m_sceneAireDeJeu = new QGraphicsScene(this);
    gv_AireDeJeu->setScene(m_sceneAireDeJeu);
    gv_AireDeJeu->setSceneRect(gv_AireDeJeu->rect());
    /* On la place sur l'AbulEduExerciceWidgetAireDeTravailV1 par l'intermédiaire d'un QGraphicsProxyWidget,
       qui va permettre de modifier l'ordre d'affichage des objets. La gv_AireDeJeu sera en dessous des autres objets */
    m_proxyAireDeJeu = getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->scene()->addWidget(gv_AireDeJeu);
    m_proxyAireDeJeu->setZValue(-1);
    gv_AireDeJeu->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    gv_AireDeJeu->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    /* On lui donne un fond transparent et pas de bordure */
    gv_AireDeJeu->setStyleSheet("background-color: rgba(0,0,0,0)");
    gv_AireDeJeu->setFrameShape(QFrame::NoFrame);
    gv_AireDeJeu->setVisible(true);
    
    /* Et on démarre la QStateMachine */
    question->assignProperty(getAbeExerciceTelecommandeV1()->ui->btnVerifier, "enabled", false);
    question->assignProperty(getAbeExerciceTelecommandeV1()->ui->btnCorriger, "enabled", false);
    sequenceMachine->start();
}

ExerciceQCM::~ExerciceQCM()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    emit exerciseExited();
}

void ExerciceQCM::setDimensionsWidgets(float ratio)
{
    /* Attention : ce slot doit être appelé une seule fois pour éviter des calculs inutiles et lourds */
    
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    /* Placement de AbulEduExerciceWidgetAireDeTravailV1 et AbulEduExerciceWidgetTelecommandeV1.
       Leur taille est recalculée grâce à leur connect avec le signal dimensionsChangees de l'AbulEduPageAccueilV1 */
    getAbeExerciceAireDeTravailV1()->move(0, 0);
    getAbeExerciceTelecommandeV1()->move(1550*ratio, 0);
    
    /* Dimensionnement et placement de l'aire de jeu
       Les paramètres en dur (250 et 100) sont liés aux contraintes de marges des images de fond */
    int haut  = getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->height() - boiteTetes->geometry().height() - 60 * ratio;
    int large = getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->width();
    if (m_localDebug) qDebug()<<"largeur"<<large<<"hauteur"<<haut;
    gv_AireDeJeu->setFixedSize(QSize(large-250 * ratio, haut - 100 * ratio));
    gv_AireDeJeu->setSceneRect(gv_AireDeJeu->rect());
    gv_AireDeJeu->move((250 * ratio) / 2, 100 * ratio);
    gv_AireDeJeu->scene()->setSceneRect(gv_AireDeJeu->rect());
    
    /* Placement des questions */
//    QList<AbulEduFlatBoutonV1*> listeBoutons = gv_AireDeJeu->scene()->findChildren<AbulEduFlatBoutonV1*>();
//    int espacement = 20;
//    if (listeBoutons.size() > 0)
//    {
//        espacement = gv_AireDeJeu->height() / listeBoutons.size();//-1
//    }
//    int abscisse = gv_AireDeJeu->width() / 3;
//    int ordonnee = 0;
//    QListIterator<AbulEduFlatBoutonV1*> iterator(listeBoutons);
//    while(iterator.hasNext())
//    {
//        AbulEduFlatBoutonV1* boutonReponse = iterator.next();
//        boutonReponse->move(abscisse,ordonnee);
//        qDebug()<<" bouton avec "<<boutonReponse->text()<<" en "<<boutonReponse->pos();
//        ordonnee += boutonReponse->height() + espacement;
////        abeApp->postEvent(boutonReponse,new QMouseEvent(QMouseEvent::HoverEnter,QPoint(abscisse+1,ordonnee+1),Qt::NoButton,Qt::NoButton,Qt::NoModifier));
//    }
    getAbeExerciceTelecommandeV1()->ui->pbarQuestion->setVisible(false);
    getAbeExerciceTelecommandeV1()->abeTelecommandeResize();
    /* Placement des têtes */
    boiteTetes->setPos((getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->width() - boiteTetes->geometry().width())/2,
                       getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->height() - boiteTetes->geometry().height() -60*ratio);
    
    /*
         Dimensionnement d'éventuels autres objets fixes de la scène
    */
    
    AbulEduCommonStatesV1::setDimensionsWidgets();
}

void ExerciceQCM::clearScene()
{
    if (m_localDebug) qDebug()<< __FILE__ <<  __LINE__ << __FUNCTION__;
    /* Le code ci-dessous suffit si gv_AireDeJeu est un QGraphicsView
       Attention : les objets graphiques présents sur la scène sont alors détruits
       Il faut également supprimer d'éventuels objets non graphiques créés à l'entrée dans l'exercice */
    gv_AireDeJeu->scene()->clear();
//    getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->scene()->clear(); // provoque une erreur de segmentation :-(
}

void ExerciceQCM::afficheQuestion()
{
    m_config->beginGroup(m_className);
        m_config->beginReadArray("exercice");
            m_config->setArrayIndex(abeStateMachineGetNumExercice());
            getAbeExerciceMessageV1()->abeWidgetMessageSetConsigne(m_config->value("texteQuestion").toString());
            m_questionPosee = m_config->value("texteQuestion").toString();
        m_config->endArray();
    m_config->endGroup();

    getAbeExerciceMessageV1()->abeWidgetMessageSetTitre(trUtf8("Question n°%1").arg(abeStateMachineGetNumExercice()+1));
    getAbeExerciceMessageV1()->setVisible(true);
}

void ExerciceQCM::afficheReponsesPossibles()
{
    int nombreReponses;
    QStringList reponses;
    m_config->beginGroup(m_className);
        m_config->beginReadArray("exercice");
        {
            m_config->setArrayIndex(abeStateMachineGetNumExercice());
                nombreReponses = m_config->beginReadArray("reponse");
                qDebug()<<" nombre réponses : "<<nombreReponses;
                for(int i = 0;i<nombreReponses;i++)
                {
                    m_config->setArrayIndex(i);
                    qDebug()<<"index "<<i<< " sur " << nombreReponses;
                    if(i == 0)
                    {
                        m_reponseAttendue = m_config->value("texteReponse").toString();
                    }
                    reponses.append(m_config->value("texteReponse").toString());
                }
                m_config->endArray();
        }
        m_config->endArray();
    m_config->endGroup();
    if(m_localDebug)
    {
        qDebug()<<m_questionPosee;
        qDebug()<<m_reponseAttendue;
        qDebug()<<reponses;
    }

    int abscisse = getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->sceneRect().width()/3;
    int ordonnee = 50;
    int espacement = 20;
    QFontMetrics fm(abeApp->font());
    QStringListIterator iter(reponses);
    while(iter.hasNext())
    {
        QString reponse = iter.next();
        AbulEduFlatBoutonV1* boutonReponse = new AbulEduFlatBoutonV1();
        connect(boutonReponse, SIGNAL(clicked()),this, SLOT(etudieClic()));
        boutonReponse->setText(reponse);

        int longueurTexte = fm.boundingRect(reponse).width();
        int hauteurTexte = fm.boundingRect(reponse).height();
        boutonReponse->resize(longueurTexte,hauteurTexte);
        boutonReponse->move(abscisse,ordonnee);
        qDebug()<<" bouton avec "<<reponse<<" en "<<boutonReponse->pos();
        ordonnee += boutonReponse->height() + espacement;
        getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->scene()->addWidget(boutonReponse);
//        gv_AireDeJeu->scene()->addWidget(boutonReponse);
    }
    setDimensionsWidgets();
}

int ExerciceQCM::verifieReponse()
{
    if(m_localDebug)
    {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "Attendu : "<<m_reponseAttendue;
        qDebug() << "Répondu : "<<m_reponseDonnee;
    }
    int nombreErreurs = 0;
    /* Calcul du nombre d'erreurs qui va servir à l'évaluation */
    m_score = QString("Score calculé").toInt();
    /* Détermination de l'évaluation parmi les valeurs possibles en fonction du nombre d'erreurs, de clics, du temps, etc... */
    if (m_reponseAttendue == m_reponseDonnee)
    {
        setAbeExerciceEvaluation(abe::evalA);
    }
    else
    {
        setAbeExerciceEvaluation(abe::evalD);
        nombreErreurs = 1;
    }
    m_tempsReponse = QString("temps mesuré");
    construisLog();
    return nombreErreurs;
}

int ExerciceQCM::donneReponse()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    m_reponseDonnee = "abandon";
    setAbeExerciceEvaluation(abe::evalZ);
    m_score = -1;
    m_tempsReponse = QString("temps mesuré");
    construisLog();
    return -1;
}

void ExerciceQCM::construisLog()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    setAbeLineLog(m_questionPosee,m_reponseDonnee,m_score,0,getAbeExerciceEvaluation(),m_reponseAttendue,m_tempsReponse);
}

void ExerciceQCM::etudieClic()
{
    AbulEduFlatBoutonV1* boutonClique = (AbulEduFlatBoutonV1*) sender();
    m_reponseDonnee = boutonClique->text();

    sequenceMachine->postEvent(new StringEvent("QuestionVerifie"));
}


/* --------------------- Implémentation spécifique des slots de l'AbulEduStateMachineV1 --------------------- */


void ExerciceQCM::slotSequenceEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__;
    
    getAbeExerciceMessageV1()->setParent(0);
    getAbeExerciceAireDeTravailV1()->ui->gvPrincipale->scene()->addWidget(getAbeExerciceMessageV1());

    abeStateMachineSetOnPeutPresenterSequence(m_config->value("ExerciceQCM/afficheConsigneSequence",false).toBool());
    /* Je vais utiliser cette variable pour décider si la question doit être présentée avant ou avec les questions
      si on veut la présenter avant - comportement par défaut - on met true et la question s'affiche dans ce qui est censé
      être la consigne d'exercice. */
    abeStateMachineSetOnPeutPresenterExercice(true);
    abeStateMachineSetOnPeutPresenterBilanExercice(true);
    
    
    /* Choix : le niveau de difficulté est le même pour tous les exercices d'une séquence */
    setAbeLevel(m_config->value("ExerciceQCM/niveauDifficulte",1).toString());
    
    QStringList listeImagesFacticePourLinstant;


    m_config->beginGroup(m_className);
    if(m_config->value("modeAutomatique").toBool() == true)
    {
        abeStateMachineSetNbExercices(m_config->value("nombreQuestionsAutomatique",qMin(10,listeImagesFacticePourLinstant.size())).toInt());
    }
    else
    {
        abeStateMachineSetNbExercices(m_config->value("exercice/size").toInt());
    }
    m_config->endGroup();

    /* Attention : la ligne ci-dessous peut être déplacée à condition de rester avant l'appel à setDimensionsWidgets(); */
    AbulEduCommonStatesV1::slotSequenceEntered();
    setDimensionsWidgets();
}

void ExerciceQCM::slotPresenteSequenceEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__;
    
    AbulEduCommonStatesV1::slotPresenteSequenceEntered();
    QString texteConsigneSequence = trUtf8("Pour chaque question, clique sur la réponse correcte.");
    m_config->beginGroup(m_className);
    if(!m_config->value("consigneSequence").toString().isEmpty())
    {
        texteConsigneSequence = m_config->value("consigneSequence").toString();
    }
    m_config->endGroup();
    /* Écriture par défaut dans le titre de présentation de séquence du nom de l'exercice
       Vous pouvez remplacer par autre chose si vous préférez */
    getAbeExerciceMessageV1()->abeWidgetMessageSetTitre(getAbeExerciceName());
    QString debutTableau = "<tr valign=middle>";
    QString imagetete = "<td> " + QString(" <img src=\":/evaluation/neutre\"></td>");
    qDebug()<<texteConsigneSequence;
    QString consigne = "<td align = right> " +  texteConsigneSequence + "</td>";
    QString finTableau = "</tr>";
    getAbeExerciceMessageV1()->abeWidgetMessageSetConsigne(debutTableau + imagetete + consigne + finTableau);
    getAbeExerciceMessageV1()->abeWidgetMessageSetZoneTexteVisible(false);
    /* Le paramètre en dur (200) est lié à une contrainte sur les marges des images de fond */
    getAbeExerciceMessageV1()->move((getAbeExerciceAireDeTravailV1()->width() - getAbeExerciceMessageV1()->width())/2,
                                    (getAbeExerciceAireDeTravailV1()->height() - getAbeExerciceMessageV1()->height())/2 - 200*abeApp->getAbeApplicationDecorRatio());
    getAbeExerciceMessageV1()->setVisible(true);
}

void ExerciceQCM::slotPresenteSequenceExited()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__;
    getAbeExerciceMessageV1()->setVisible(false);
}

void ExerciceQCM::slotPresentationExerciceEntered()
{
    if (m_localDebug) qDebug()<< __FILE__ <<  __LINE__ << __FUNCTION__;
    /** @todo Documenter une structure type de fichier de configuration pour les modules */

    afficheQuestion();
    
    AbulEduCommonStatesV1::slotPresentationExerciceEntered();
}

void ExerciceQCM::slotPresentationExerciceExited()
{
    if (m_localDebug) qDebug()<< __FILE__ <<  __LINE__ << __FUNCTION__;
    getAbeExerciceMessageV1()->setVisible(false);
}

void ExerciceQCM::slotRealisationExerciceEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    setAbeNbTotalQuestions(1);

    AbulEduCommonStatesV1::slotRealisationExerciceEntered();
    if (getAbeNbTotalQuestions() > 1)
    {
        getAbeExerciceTelecommandeV1()->ui->pbarQuestion->setVisible(true);
    }
    else
    {
        getAbeExerciceTelecommandeV1()->ui->pbarQuestion->setVisible(false);
    }
}

void ExerciceQCM::slotInitQuestionEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    abeStateMachineSetNbClicSurVerifier(0);
    clearScene();
    /*
        Lecture dans le fichier de configuration de paramètres nécessaires pour afficher la question abeStateMachineGetNumQuestion()
        Construction ou affichage d'objets communs à toutes les questions
    */
    AbulEduCommonStatesV1::slotInitQuestionEntered();
 }

void ExerciceQCM::slotQuestionEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    if (onPeutPresenterExercice)
    {
        getAbeExerciceMessageV1()->setVisible(false);
    }
    else
    {
        afficheQuestion();
    }
    afficheReponsesPossibles();

    AbulEduCommonStatesV1::slotQuestionEntered();
}

void ExerciceQCM::slotAfficheVerificationQuestionEntered()
{
    abeStateMachineSetNbClicSurVerifier(abeStateMachineGetNbClicSurVerifier()+1);
    if (m_localDebug) qDebug()<< __FILE__ <<  __LINE__ << __FUNCTION__<<trUtf8("::Nombre de clics sur vérifier : ")<<abeStateMachineGetNbClicSurVerifier();
    /* Choix : la fonction verifieReponse() retourne le nombre d'erreurs dans la réponse */
    m_verifieReponse = verifieReponse();
    setHelpText(trUtf8("Message à afficher par l'aide suite à la vérification, plusieurs cas éventuellement possibles..."));
    if (m_verifieReponse == 0)
        id_temporisation = sequenceMachine->postDelayedEvent(new StringEvent("finAfficheVerification"), 10000);
    else
        sequenceMachine->postEvent(new StringEvent("finAfficheVerification"));
}

void ExerciceQCM::slotFinVerificationQuestionEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    sequenceMachine->cancelDelayedEvent(id_temporisation);
    sequenceMachine->postEvent(new StringEvent("Questionsdone"));
}

void ExerciceQCM::slotAfficheCorrectionQuestionEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    setHelpText(trUtf8("Message à afficher par l'aide suite à la correction."));
    setAbeTeteTelecommande("rien");
    m_verifieReponse = donneReponse();
    id_temporisation = sequenceMachine->postDelayedEvent(new StringEvent("finAfficheCorrection"), 10000);
}

void ExerciceQCM::slotFinCorrectionQuestionEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    sequenceMachine->cancelDelayedEvent(id_temporisation);
    sequenceMachine->postEvent(new StringEvent("QuestionsDoneCorrect"));
}

void ExerciceQCM::slotBilanExerciceEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    clearScene();
    
    if(m_verifieReponse == -1)
    {
        m_resultats.append(QPair<int, int>(m_numExercice, -1));
    }
    else
    {
        m_resultats.append(QPair<int, int>(m_numExercice, abeStateMachineGetNbClicSurVerifier()));
    }
    // On affiche la bonne tete en fonction du nb de clics sur btnVerifier
    QString imagetete = ":/bilan/";
    
    /* Choix : évaluation fixée à Très bien si réussite au premier essai
                évaluation fixée à Bien si réussite en deux essais (1 erreur seulement)
                évaluation fixée à Moyen si réussite en trois ou quatre essais
                évaluation fixée à Mal au-delà
        Vous pouvez adapter en déplaçant les instructions dans un autre case, ou en en rajoutant.
    */
    switch(m_verifieReponse)
    {
    case -1: // Abandon
        boiteTetes->setEtatTete(m_numExercice, abe::evalZ);
        imagetete.append(boiteTetes->suffixe->value(abe::evalZ));
        break;
    case 0:
        boiteTetes->setEtatTete(m_numExercice, abe::evalA);
        imagetete.append(boiteTetes->suffixe->value(abe::evalA));
        break;
    case 1:
        boiteTetes->setEtatTete(m_numExercice, abe::evalD);
        imagetete.append(boiteTetes->suffixe->value(abe::evalD));
        break;
    default:
        boiteTetes->setEtatTete(m_numExercice, abe::evalD);
        imagetete.append(boiteTetes->suffixe->value(abe::evalD));
    }
    
    AbulEduCommonStatesV1::slotBilanExerciceEntered();
    QString message;
    if (m_verifieReponse == -1)
    {
        boiteTetes->setEtatTete(m_numExercice, abe::evalZ);
        imagetete = ":/bilan/"+boiteTetes->suffixe->value(abe::evalZ);
        if(abeStateMachineGetNbClicSurVerifier()==0)
        {
            message = trUtf8("Correction demandée sans avoir proposé de réponse");
        }
        else if (abeStateMachineGetNbClicSurVerifier()==1)
        {
            message = trUtf8("Correction demandée après %1 proposition de réponse").arg(abeStateMachineGetNbClicSurVerifier());
        }
        else
        {
            message = trUtf8("Correction demandée après %1 propositions de réponse").arg(abeStateMachineGetNbClicSurVerifier());
        }
    }
    else
    {
        if(m_verifieReponse == 0)
        {
            message = trUtf8("Bravo, c'était la bonne réponse !");
        }
        else
        {
            message = trUtf8("Ah non ! Ce n'est pas la bonne réponse...");
        }
    }
    if (m_config->value(m_className+"/exercice/"+QString::number(m_numExercice + 1)+"/titre").toString().isEmpty())
    {
        getAbeExerciceMessageV1()->abeWidgetMessageSetTitre(m_className+" "+QString::number(m_numExercice + 1));
    }
    else
    {
        getAbeExerciceMessageV1()->abeWidgetMessageSetTitre(m_config->value(m_className+"/exercice/"+QString::number(m_numExercice + 1)+"/titre").toString());
    }
    getAbeExerciceMessageV1()->abeWidgetMessageSetConsigne(QString("<center><img src=")+imagetete+QString("/><br>")
                                                           + message + QString("</center>"));
    getAbeExerciceMessageV1()->abeWidgetMessageResize();
    getAbeExerciceMessageV1()->abeWidgetMessageSetZoneTexteVisible(false);
    
    getAbeExerciceMessageV1()->move((getAbeExerciceAireDeTravailV1()->width() - getAbeExerciceMessageV1()->width())/2,
                                    ((getAbeExerciceAireDeTravailV1()->height() - getAbeExerciceMessageV1()->height())/2) - 200*abeApp->getAbeApplicationDecorRatio());
    getAbeExerciceMessageV1()->setVisible(true);
    
}

void ExerciceQCM::slotBilanSequenceEntered()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    AbulEduCommonStatesV1::slotBilanSequenceEntered();
    QString message = "";
    int tailleImageTete = QPixmap(":/bilan/"+boiteTetes->suffixe->value(0)).width() * abeApp->getAbeApplicationDecorRatio();
    message = "<center>" + message;
    int numExercice = 0;
    int numExerciceAtraiter = 0;
    int numtete = 0;
    QPair<int, int> resultat;
    message = message + QString("<table cellpadding=\"4\" cellspacing=\"0\" border=\"0\" align=\"center\" valign=\"middle\"><tr><td>") + m_config->value("sequence/exercice/"+QString::number(resultat.first +1 )+"/titre").toString()+ QString("</td>");
    /* Pour chacun des éléments de la liste m_resultats */
    foreach(resultat, m_resultats)
    {
        /* On récupère le numéro de l'exercice */
        numExerciceAtraiter = resultat.first;
        /* Si on a changé d'exercice on crée une ligne */
        if(numExercice != numExerciceAtraiter)
        {
            QString titreExercice;
            if (m_config->value(m_className+"/exercice/"+QString::number(m_numExercice + 1)+"/titre").toString().isEmpty())
            {
                titreExercice = m_config->value(m_className+" "+QString::number(m_numExercice + 1)).toString();
            }
            else
            {
                titreExercice = m_config->value(m_className+"/exercice/"+QString::number(m_numExercice + 1)+"/titre").toString();
            }
            numExercice = numExerciceAtraiter;
            message = message +"</tr>"+QString("<tr><td>")
                    + titreExercice+ QString("</td>");
        }
        
        switch(resultat.second)
        {
        case -1:
            numtete = abe::evalZ;
            break;
        case 0:
        case 1:
            numtete = abe::evalA;
            break;
        case 2:
            numtete = abe::evalB;
            break;
        case 3:
        case 4:
            numtete = abe::evalC;
            break;
        default:
            numtete = abe::evalD;
        }
        QString imagetete = "<td> " + QString(" <img src=\":/bilan/"+boiteTetes->suffixe->value(numtete)+"\" width=%1/></td>")
                .arg(tailleImageTete);
        message = message + imagetete;
        
        if (m_localDebug) qDebug()<<"code pour l'exercice "<<numExercice<<" : "<<message;
    }
    
    message = message+"</table";
    getAbeExerciceMessageV1()->abeWidgetMessageSetTitre(getAbeExerciceName());
    getAbeExerciceMessageV1()->abeWidgetMessageSetConsigne(message);
    getAbeExerciceMessageV1()->abeWidgetMessageResize();
    getAbeExerciceMessageV1()->abeWidgetMessageSetZoneTexteVisible(false);
    getAbeExerciceMessageV1()->move((getAbeExerciceAireDeTravailV1()->width() - getAbeExerciceMessageV1()->width())/2,
                                    ((getAbeExerciceAireDeTravailV1()->height() - getAbeExerciceMessageV1()->height())/2) - 200*abeApp->getAbeApplicationDecorRatio());
    getAbeExerciceMessageV1()->setVisible(true);
}

void ExerciceQCM::slotQuitter()
{
    if(m_localDebug) qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    AbulEduCommonStatesV1::slotQuitter();
}

