/** Classe de base pour les mainwindows des logiciels AbulEdu (Leterrier-QCM)
  *
  * @see https://redmine.ryxeo.com/projects/leterrier-qcm
  * @author 2013 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * 
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/LeTerrierMainWindow
  * @author 2012 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * @author 2012 Éric Seigne <eric.seigne@ryxeo.com>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "leterriermainwindow.h"
#include "ui_leterriermainwindow.h"

LeTerrierMainWindow::LeTerrierMainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::LeTerrierMainWindow)
{
    m_localDebug = DEBUG_LETERRIERMAINWINDOW;
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__<<"-----Constructeur";
    ui->setupUi(this);
    createLangMenu();
    
    /* Initialisations */
    m_abuleduFile = QSharedPointer<AbulEduFileV1>(new AbulEduFileV1, &QObject::deleteLater);
    //    m_abuleduFileManager = new AbulEduBoxFileManagerV1(0);
    ui->abeBoxFileManager->abeSetFile(m_abuleduFile);
    connect(ui->abeBoxFileManager, SIGNAL(signalAbeFileSelected()), this, SLOT(slotOpenFile()), Qt::UniqueConnection);
    connect(ui->abeBoxFileManager, SIGNAL(signalAbeFileSaved(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation,QString,bool)), this, SLOT(slotAfficheEtatEnregistrement(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation,QString,bool)), Qt::UniqueConnection);

    m_nam = abeApp->getAbeNetworkAccessManager();
    connect(m_nam,SIGNAL(ssoAuthStatus(int)),this,SLOT(slotSSOStatusChanged(int)), Qt::UniqueConnection);
    m_exerciseNumber = -1;
    m_isExerciseRunning = false;
    m_isEditorRunning = false;
    m_isDemoAvailable = true;
    
    /* Chargement langue */
    QString locale = QLocale::system().name();
    m_qtTranslator.load("qt_" + locale,
                        QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    abeApp->installTranslator(&m_qtTranslator);
    m_myAppTranslator.load(abeApp->applicationName() +"_"+ locale, "lang");
    abeApp->installTranslator(&m_myAppTranslator);
    
    /* Installation de la page d'accueil */
    m_speechBubbles = new QMap<int, QString>();
    /* Ajouter pour chaque type d'exercice un nom destiné à la bulle de la zone de lancement, sur le modèle */
    m_speechBubbles->insert(1,"Quizz");
    
    m_abuleduPageAccueil = new AbulEduPageAccueilV1(m_speechBubbles, ui->pageAccueil);
    connect(m_abuleduPageAccueil, SIGNAL(boutonPressed(int)), this, SLOT(slotAbeLaunchExercise(int)), Qt::UniqueConnection);
    connect(m_abuleduPageAccueil->abePageAccueilGetMenu(), SIGNAL(btnQuitterTriggered()), this, SLOT(on_actionQuitter_triggered()), Qt::UniqueConnection);
    connect(m_abuleduPageAccueil->abePageAccueilGetMenu(), SIGNAL(btnBoxTriggered()), this, SLOT(on_action_Ouvrir_un_module_triggered()), Qt::UniqueConnection);
    connect(m_abuleduPageAccueil->abePageAccueilGetBtnRevenirEditeur(), SIGNAL(clicked()),this,SLOT(backToEditor()), Qt::UniqueConnection);
    
    if (m_speechBubbles->size() > 0)
    {
        QMapIterator<int, QString> iterator(*m_speechBubbles);
        while(iterator.hasNext())
        {
            iterator.next();
            QAction* actionExercice = new QAction(iterator.value(),this);
            actionExercice->setObjectName(QString::number(iterator.key()));
            actionExercice->setShortcut(QKeySequence("Ctrl+"+actionExercice->objectName()));
            connect(actionExercice, SIGNAL(triggered()), this, SLOT(slotAbeLaunchExercise()), Qt::UniqueConnection);
            ui->menuExercices->addAction(actionExercice);
        }
    }
    
    connect(ui->widgetEditeur, SIGNAL(openModuleLoaderForEditeur()), this, SLOT(openModuleLoader()));
    connect(ui->widgetEditeur, SIGNAL(openModuleUpLoaderForEditeur()), this, SLOT(openModuleUpLoader()));
    
    /* La boite à propos, même en version V1, n'est pas une lib externe, je ne l'inclus donc pas pour l'instant */
    //        AbulEduAproposV1 *monAide=new AbulEduAproposV1(this); //Pour plus tard
    //    connect(monAide, SIGNAL(btnAideClicked()), this, SLOT(afficheAide()));
    //    monAide->hide();
    /* Gestion du mode tablette automatique */
#ifdef __ABULEDUTABLETTEV1__MODE__
    ui->menuBar->hide();
    ui->mainToolBar->hide();
    ui->statusBar->hide();
#else
    /* On met un titre à la fenêtre */
    setTitle(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus());
#endif
    // On redimensionne les widgets
    m_abuleduPageAccueil->setDimensionsWidgets();
}

LeTerrierMainWindow::~LeTerrierMainWindow()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__<<"-----Destructeur";
    delete ui;
}

void LeTerrierMainWindow::resizeEvent(QResizeEvent *)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    // On redimensionne les widgets
    m_abuleduPageAccueil->setDimensionsWidgets();
}

void LeTerrierMainWindow::abeSetMyAbulEduFile(QSharedPointer<AbulEduFileV1> file)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_abuleduFile = file;
}

QSharedPointer<AbulEduFileV1> LeTerrierMainWindow::abeGetMyAbulEduFile()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    return m_abuleduFile;
}

AbulEduPageAccueilV1 *LeTerrierMainWindow::abeGetMyAbulEduAccueil()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    return m_abuleduPageAccueil;
}

void LeTerrierMainWindow::createLangMenu()
{
    QString locale = QLocale::system().name().split("_").first();
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__<<" !! "<<locale;
    QAction* actionLangueEn = new QAction(trUtf8("English"),this);
    actionLangueEn->setObjectName("en");
    actionLangueEn->setCheckable(true);
    if(actionLangueEn->objectName() == locale)
    {
        actionLangueEn->setChecked(true);
    }
    connect(actionLangueEn, SIGNAL(triggered()), this, SLOT(slotChangeLang()),Qt::UniqueConnection);
    ui->menuLangues->addAction(actionLangueEn);
    
    QAction* actionLangueFr = new QAction(trUtf8("Français"),this);
    actionLangueFr->setObjectName("fr");
    actionLangueFr->setCheckable(true);
    if(actionLangueFr->objectName() == locale)
    {
        actionLangueFr->setChecked(true);
    }
    connect(actionLangueFr, SIGNAL(triggered()), this, SLOT(slotChangeLang()),Qt::UniqueConnection);
    ui->menuLangues->addAction(actionLangueFr);
    
    /* Décommenter pour ajouter une ligne au menu Langues
    QAction* actionLangueEs = new QAction(trUtf8("Español"),this);
    actionLangueEs->setObjectName("es");
    actionLangueEs->setCheckable(true);
    if(actionLangueEs->objectName() == locale)
    {
        actionLangueEs->setChecked(true);
    }
    connect(actionLangueEs, SIGNAL(triggered()), this, SLOT(slotChangeLangue()),Qt::UniqueConnection);
    ui->menuLangues->addAction(actionLangueEs); */
    
    /* Décommenter pour ajouter une ligne au menu Langues
    QAction* actionLangueDe = new QAction(trUtf8("Deutsche"),this);
    actionLangueDe->setObjectName("de");
    actionLangueDe->setCheckable(true);
    if(actionLangueDe->objectName() == locale)
    {
        actionLangueDe->setChecked(true);
    }
    connect(actionLangueDe, SIGNAL(triggered()), this, SLOT(slotChangeLangue()),Qt::UniqueConnection);
    ui->menuLangues->addAction(actionLangueDe); */
    
    /* Décommenter pour ajouter une ligne au menu Langues
    QAction* actionLangueIt = new QAction(trUtf8("Italiano"),this);
    actionLangueIt->setObjectName("it");
    actionLangueIt->setCheckable(true);
    if(actionLangueIt->objectName() == locale)
    {
        actionLangueIt->setChecked(true);
    }
    connect(actionLangueIt, SIGNAL(triggered()), this, SLOT(slotChangeLangue()),Qt::UniqueConnection);
    ui->menuLangues->addAction(actionLangueIt); */
    
    /* Décommenter pour ajouter une ligne au menu Langues
    QAction* actionLangueAr = new QAction(trUtf8("عربي"),this);
    actionLangueAr->setObjectName("ar");
    actionLangueAr->setCheckable(true);
    if(actionLangueAr->objectName() == locale)
    {
        actionLangueAr->setChecked(true);
    }
    connect(actionLangueAr, SIGNAL(triggered()), this, SLOT(slotChangeLangue()),Qt::UniqueConnection);
    ui->menuLangues->addAction(actionLangueAr); */
}

void LeTerrierMainWindow::slotAbeLaunchExercise(int number)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_abuleduPageAccueil->abePageAccueilDesactiveZones(true);
    if (!sender()->objectName().isEmpty())
    {
        m_exerciseNumber = sender()->objectName().toInt();
    }
    else
    {
        m_exerciseNumber = number;
        ui->stackedWidget->setCurrentWidget(ui->pageBoxFileManager);
    }
    //    setFixedSize(width(),height());
    
    /* Créer des exercices "exempleExo", dont le type dépend de number
       puis
       connect(exempleExo, SIGNAL(exerciceExited()), this, SLOT(abeBackFromExercice()), Qt::UniqueConnection); */
    ExerciceQCM *ex = new ExerciceQCM(m_abuleduPageAccueil,m_abuleduFile->abeFileGetDirectoryTemp().absolutePath());
        connect(ex, SIGNAL(exerciseExited()), this, SLOT(slotAbeBackFromExercise()), Qt::UniqueConnection);


    switch(number)
    {
    case 0:
    {
        setTitle(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus());
        m_isDemoAvailable = false;
        m_abuleduPageAccueil->abePageAccueilGetBtnRevenirEditeur()->setVisible(false);
        
        break;
    }
    case 1:
        m_abuleduPageAccueil->abePageAccueilGetBtnRevenirEditeur()->setVisible(false);
        break;
    default:
        break;
    }
    setTitle(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus());
}


void LeTerrierMainWindow::slotAbeBackFromExercise()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_abuleduPageAccueil->abePageAccueilDesactiveZones(false);
    m_exerciseNumber = -1;
    if(m_isEditorRunning)
    {
        m_abuleduPageAccueil->abePageAccueilGetBtnRevenirEditeur()->setVisible(true);
    }
    setFixedSize(QSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX));
    m_isDemoAvailable = ui->actionMode_D_mo->isChecked();
    setTitle(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus());
}

void LeTerrierMainWindow::slotChangeLang()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    foreach(QAction* act,ui->menuLangues->actions())
    {
        act->setChecked(false);
    }
    QString lang = "en";
    if (sender())
    {
        lang = sender()->objectName();
        static_cast<QAction *>(sender())->setChecked(true);
    }
    
    abeApp->removeTranslator(&m_qtTranslator);
    abeApp->removeTranslator(&m_myAppTranslator);
    
    m_qtTranslator.load("qt_" + lang, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    abeApp->installTranslator(&m_qtTranslator);
    
    m_myAppTranslator.load(abeApp->applicationName() + "_" + lang, "lang");
    abeApp->installTranslator(&m_myAppTranslator);
    ui->retranslateUi(this);
}


void LeTerrierMainWindow::slotOpenFile()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_abuleduFile = ui->abeBoxFileManager->abeGetFile();
    if(m_exerciseNumber > 0)
    {
        ui->stackedWidget->setCurrentWidget(ui->pageAccueil);
        slotAbeLaunchExercise(m_exerciseNumber);
    }
    else if(m_exerciseNumber == 0)
    {
        ui->stackedWidget->setCurrentWidget(ui->pageEditeur);
    }
    else
    {
        ui->stackedWidget->setCurrentWidget(ui->pageAccueil);
    }
    setTitle(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus());
}

void LeTerrierMainWindow::slotAfficheEtatEnregistrement(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation loc, QString nom, bool reussite)
{
    QString emplacement;
    if (loc == AbulEduBoxFileManagerV1::abePC)
    {
        emplacement = trUtf8("votre ordinateur");
    }
    else if (loc == AbulEduBoxFileManagerV1::abeBoxPerso)
    {
        emplacement = trUtf8("votre abeBox personnelle");
    }
    else if (loc == AbulEduBoxFileManagerV1::abeBoxShare)
    {
        emplacement = trUtf8("une abeBox partagée");
    }
    else if (loc == AbulEduBoxFileManagerV1::abeMediatheque)
    {
        emplacement = trUtf8("AbulEdu-Médiathèque");
    }
    else
    {
        emplacement = trUtf8("un endroit inconnu");
    }

    QString message;
    if (reussite == true)
    {
        message = trUtf8("Votre fichier a été enregistré dans ")+emplacement;
        if (!nom.isEmpty())
        {
            message.append(trUtf8(" sous le nom : ")+nom.split("/").last());
        }
    }
    else
    {
        message = trUtf8("Votre fichier n'a pas pu être enregistré...");
    }
    AbulEduMessageBoxV1* msgEnregistrement = new AbulEduMessageBoxV1(trUtf8("Enregistrement"), message);
    if (reussite == true)
    {
        msgEnregistrement->setWink();
    }
    msgEnregistrement->show();
    ui->stackedWidget->setCurrentWidget(ui->pageAccueil);
}

void LeTerrierMainWindow::setTitle(int authStatus)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    QString title = abeApp->getAbeApplicationLongName();
    
    if (m_exerciseNumber >0)
    {
        title.append(" -- " + m_speechBubbles->value(m_exerciseNumber));
    }
    
    
    if(!m_abuleduFile->abeFileGetFileName().fileName().isEmpty())
    {
        title.append(" -- "+m_abuleduFile->abeFileGetFileName().fileName());
    }
    if (authStatus == 1)
    {
        title.append(" -- "+abeApp->getAbeNetworkAccessManager()->abeGetSSOLogin());
    }
    setWindowTitle(title);
}

void LeTerrierMainWindow::slotSSOStatusChanged(int code)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    /* Insérez ici le code que vous voulez concernant la modification d'état de l'authentification
       code = -1 (erreur d'authentification), 0 (on ne sait pas, jamais tenté), ou 1 (authentifié) */
    setTitle(code);
}

void LeTerrierMainWindow::slotSessionAuthenticated(bool enable)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    if(enable)
    {
        m_nam->abeSSOLogin();
    }
    
    activityFilter *ef;
    ef = new activityFilter(abeApp);
    ef->setInterval(7000);
    abeApp->installEventFilter(ef);
    QObject::connect(ef, SIGNAL( userInactive() ),
                     this,  SLOT( slotDemo() ), Qt::UniqueConnection);
    connect(m_nam, SIGNAL(ssoAuthStatus(int)), this,SLOT(setTitle(int)), Qt::UniqueConnection);
}

void LeTerrierMainWindow::slotDemo()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    if (m_isDemoAvailable == false)
    {
        return;
    }
    QKeyEvent* pressSpace = new QKeyEvent(QEvent::KeyPress,Qt::Key_Space,Qt::NoModifier);
    QApplication::sendEvent(m_abuleduPageAccueil,pressSpace);
    m_abuleduMessageBox = new AbulEduMessageBoxV1(trUtf8("On y va ?"),trUtf8("Clique sur une zone pour lancer un exercice"),this);
    m_abuleduMessageBox->setWink();
    m_abuleduMessageBox->hideCloseButton();
    m_abuleduMessageBox->show();
    foreach(AbulEduZoneV1* zone, m_abuleduPageAccueil->abePageAccueilGetZones())
    {
        zone->abeZoneDrawRect(true,QColor("#dcdcdc"));
    }
    m_abuleduPageAccueil->setDimensionsWidgets();
    QTimer::singleShot(4000,this,SLOT(slotFinDemo()));
}

void LeTerrierMainWindow::slotFinDemo()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_abuleduMessageBox->close();
    foreach(AbulEduZoneV1* zone, m_abuleduPageAccueil->abePageAccueilGetZones())
    {
        zone->abeZoneDrawRect(false);
    }
    QKeyEvent* releaseSpace = new QKeyEvent(QEvent::KeyRelease,Qt::Key_Space,Qt::NoModifier);
    QApplication::sendEvent(m_abuleduPageAccueil,releaseSpace);
}

void LeTerrierMainWindow::on_action_Ouvrir_un_module_triggered()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    if(!m_isEditorRunning)
    {
        ui->stackedWidget->setCurrentWidget(ui->pageBoxFileManager);
    }
}

void LeTerrierMainWindow::on_action_Editeur_de_modules_triggered()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    
    if (!m_isExerciseRunning) // si on est en cours exercice pas d'éditeur
    {
        ui->stackedWidget->setCurrentWidget(ui->pageEditeur);
        ui->widgetEditeur->abeEditeurSetMainWindow(this);
        ui->action_Editeur_de_modules->setEnabled(false);
        ui->action_Ouvrir_un_module->setEnabled(false);
        connect(ui->widgetEditeur, SIGNAL(editeurClosed()), this, SLOT(editorClosed()));
        connect(ui->widgetEditeur,
                SIGNAL(tryThisSettings(QSharedPointer<AbulEduFileV1>)),
                this,
                SLOT(tryNewSettings(QSharedPointer<AbulEduFileV1>)));
        
        m_isExerciseRunning = true;
        m_isEditorRunning = true;
        m_isDemoAvailable = false;
    }
    else // On affiche un petit message...
    {
        QMessageBox::critical(this,"Ouverture Editeur", trUtf8("Veuillez quitter l'exercice avant d'ouvrir l'éditeur"),0,0);
    }
}

void LeTerrierMainWindow::on_actionQuitter_triggered()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    close();
}

void LeTerrierMainWindow::on_action_Journal_de_mes_activit_s_triggered()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    abeApp->getAbeIdentite()->abeGetMyLogsAsPDF();
}

void LeTerrierMainWindow::on_action_Authentification_triggered()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_nam->abeSSOLogout();
    m_nam->abeSSOLogin();
}
void LeTerrierMainWindow::showEvent ( QShowEvent * event )
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    Q_UNUSED(event)
#ifdef __ABULEDUTABLETTEV1__MODE__
    showFullScreen();
#else
    showMaximized();
#endif
}

void LeTerrierMainWindow::openModuleLoader()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_exerciseNumber = 0;
    ui->stackedWidget->setCurrentWidget(ui->pageBoxFileManager);
}

void LeTerrierMainWindow::openModuleUpLoader()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_exerciseNumber = 0;
    ui->abeBoxFileManager->abeSetOpenOrSaveEnum(AbulEduBoxFileManagerV1::abeSave);
    ui->abeBoxFileManager->abeSetFile(m_abuleduFile);
    ui->stackedWidget->setCurrentWidget(ui->pageBoxFileManager);
}



void LeTerrierMainWindow::tryNewSettings(QSharedPointer<AbulEduFileV1> file)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__<<file;
    m_abuleduFile.clear();
    m_abuleduFile=file;
    m_isExerciseRunning=false;
    ui->stackedWidget->setCurrentWidget(ui->pageAccueil);
    m_abuleduPageAccueil->abePageAccueilGetBtnRevenirEditeur()->setVisible(true);
}

void LeTerrierMainWindow::backToEditor()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_isExerciseRunning=true;
    m_abuleduPageAccueil->abePageAccueilGetBtnRevenirEditeur()->setVisible(false);
    ui->stackedWidget->setCurrentWidget(ui->pageEditeur);
}
void LeTerrierMainWindow::editorClosed()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_isExerciseRunning = false;
    ui->action_Ouvrir_un_module->setEnabled(true);
    ui->action_Editeur_de_modules->setEnabled(true);
    ui->stackedWidget->setCurrentWidget(ui->pageAccueil);
}

