/**
  * Classe de base en version une pour les exercices AbulEdu permettant de structurer
  * nos logiciels et respecter ainsi un certain nombre de choses
  * @author 2009-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GPL)
  *
  */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "abuleduexercicev1.h"

/** constructeur ... */
AbulEduExerciceV1::AbulEduExerciceV1(QWidget *parent) : QObject(parent), AbulEduExerciceCommonV1(this)
{
    /* Initialisation du générateur de nombres aléatoires */
    #if QT_VERSION >= 0x040700
        qsrand(uint(QDateTime::currentMSecsSinceEpoch() & 0xFFFF));
    #else
        QDateTime t;
        qsrand(uint(t.toTime_t() & 0xFFFF));
    #endif

    m_localDebug = false;

    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    setObjectName("AbulEduExerciceV1");
    setProperty("AbulEduType","AbulEduExerciceV1");
    //Les themes par defaut
    m_themeTetes = "asma";
    m_themeLevel = "chiffres";

    m_uiAbulEduAireDeTravail = new AbulEduExerciceWidgetAireDeTravailV1(parent);
    m_uiAbulEduTelecommande = new AbulEduExerciceWidgetTelecommandeV1(parent);
    m_uiAbulEduMessage = new AbulEduExerciceWidgetMessageV1(parent);

    // Pas besoin de montrer les AireDeTravail et Telecommande ici (évite l'affichage multiple des images)
    m_uiAbulEduAireDeTravail->setVisible(false);
    m_uiAbulEduTelecommande->setVisible(false);
    m_uiAbulEduMessage->setVisible(false);

    //On "force" la taille de la fenêtre selon les contraintes abuledu
    //setGeometry(1,1,1024,550);

    //objets de l'interface graphiques
    m_uiAbulEduAireDeTravail->ui->gvPrincipale->setCacheMode(QGraphicsView::CacheBackground);
    m_uiAbulEduAireDeTravail->ui->gvPrincipale->setOptimizationFlag(QGraphicsView::DontAdjustForAntialiasing);
    //    connect(ui->btnAccueilSuite, SIGNAL(clicked()), this, SLOT(on_btnAccueilSuite_clicked()));

}

AbulEduExerciceV1::~AbulEduExerciceV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
}

void AbulEduExerciceV1::setAbeLevel(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    AbulEduExerciceCommonV1::setAbeLevel(s);
    m_uiAbulEduTelecommande->setAbeLevel(s);
}

void AbulEduExerciceV1::setAbeNbTotalQuestions(int n)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    AbulEduExerciceCommonV1::setAbeNbTotalQuestions(n);
    m_uiAbulEduTelecommande->ui->pbarQuestion->setRange(0, getAbeNbTotalQuestions());
    m_uiAbulEduTelecommande->ui->pbarQuestion->update();
}

void AbulEduExerciceV1::setAbeTeteTelecommande(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Attention il faudra vérifier que s = tresbien, rien, moyen, mal, bien, attente
    QStringList vocabulaire;
    vocabulaire << "tresbien" << "rien" << "moyen" << "mal" << "bien" << "attente";
    if(s.trimmed() != "" && vocabulaire.contains(s)) {
        m_uiAbulEduTelecommande->ui->lblTetePersonnage->setPixmap(QPixmap(":/abuleduexercicev1/tetes/"  + s));
        m_uiAbulEduTelecommande->setMTeteEval(s);
    }
    else {
        qWarning() << "Attention vous devez passer un mot du vocabulaire à l'appel de la fonction AbulEduExerciceV1::setAbeTeteTelecommande " << vocabulaire;
    }
}

qreal AbulEduExerciceV1::setAbeTeteForResult(int errors, int total)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    qreal percentage = -1;
    if (total == 0)
    {
        return percentage;
    }
    else
    {
        QSettings config(":/abuleduexercicev1/conf/abuleduexercicev1.conf",QSettings::IniFormat);
        int veryGoodPoint = 10;
        if (config.value("seuilTresBien").isValid())
            veryGoodPoint = config.value("seuilTresBien").toInt();
        int goodPoint = 30;
        if (config.value("seuilBien").isValid())
            goodPoint = config.value("seuilBien").toInt();
        int mediumPoint = 60;
        if (config.value("seuilMoyen").isValid())
            mediumPoint = config.value("seuilMoyen").toInt();

        qreal errorsAsReal = static_cast<qreal> (errors);
        percentage = (errorsAsReal/total) * 100;

        if(m_localDebug) {
            qDebug() << "  " << __FUNCTION__ << " Pourcentage d'erreurs : " << percentage;
        }
        if (percentage <= veryGoodPoint)
        {
            setAbeTeteTelecommande("tresbien");
            setAbeExerciceEvaluation(abe::evalA);
        }
        else if (percentage <= goodPoint)
        {
            setAbeTeteTelecommande("bien");
            setAbeExerciceEvaluation(abe::evalB);
        }
        else if (percentage <= mediumPoint)
        {
            setAbeTeteTelecommande("moyen");
            setAbeExerciceEvaluation(abe::evalC);
        }
        else
        {
            setAbeTeteTelecommande("mal");
            setAbeExerciceEvaluation(abe::evalD);
        }
    }
    return percentage;
}

void AbulEduExerciceV1::setAbeThemeTeteTelecommande(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    qFatal("FATAL: DEPRECATED AbulEduExerciceV1::setAbeThemeTeteTelecommande");
    //remplacé par les qrc qui embarquent leur 'theme'
    m_themeTetes = s;
}

void AbulEduExerciceV1::setAbeThemeLevelTelecommande(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    qFatal("FATAL: DEPRECATED AbulEduExerciceV1::setAbeThemeLevelTelecommande");
    //remplacé par les qrc qui embarquent leur 'theme'
    m_themeLevel = s;
}

AbulEduExerciceWidgetTelecommandeV1 * AbulEduExerciceV1::getAbeExerciceTelecommandeV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_uiAbulEduTelecommande;
}

AbulEduExerciceWidgetAireDeTravailV1 * AbulEduExerciceV1::getAbeExerciceAireDeTravailV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_uiAbulEduAireDeTravail;
}

AbulEduExerciceWidgetMessageV1 *AbulEduExerciceV1::getAbeExerciceMessageV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_uiAbulEduMessage;
}

void AbulEduExerciceV1::slotAbeFermerExercice()
{
    m_uiAbulEduTelecommande->on_btnQuitterRetourMenuPrincipal_clicked();
}

void AbulEduExerciceV1::displayErrorMissingExercise(QString nomExercice)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    AbulEduMessageBoxV1* messageBox = new AbulEduMessageBoxV1(trUtf8("Exercice absent du module"),trUtf8("Ce module ne contient aucun exercice de type ")+"<i>"+nomExercice+"</i>");
    messageBox->show();
}
