/**
  * Classe de base en version une pour les exercices AbulEdu permettant de structurer
  * nos logiciels et respecter ainsi un certain nombre de choses dont l'interface publique
  * à laquelle le plugin peut se connecter pour échanger des données (logs pour le moment).
  * La classe AbulEduExerciceV1 hérite de QMainWindow pour que vous puissiez créer une fenêtre
  * qui dérive d'AbulEduExerciceV1 le plus simplement possible.
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/Normalisation_des_%C3%A9changes_entre_l%27application_et_le_plugin
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/Utilisation_de_abuleduexercicev1
  * @author 2009-2011 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GPL)
  *
  * @warning NE MODIFIEZ SURTOUT PAS CES FICHIERS, SI VOUS LE FAITES NE SOYEZ PAS ÉTONNÉS
  *          QUE L'APPLICATION NE S'INTERFACE PLUS CORRECTEMENT AVEC LE PLUGIN !!!!!
  *
  * #AbulEduExerciceV1
  * include(lib/abuleduexercicev1/abuleduexercicev1.pri)
  *
  */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef ABULEDUEXERCICEV1_H
#define ABULEDUEXERCICEV1_H

#include <QApplication>
#include <QDebug>
#include <QPluginLoader>
#include <QDir>
#include <QWidget>
#include <QSettings>
#include <QString>
#include <QDate>
#include <QHash>
#include <QEvent>
#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkRequest>
#include <QtNetwork/QNetworkReply>
#include <QtXml/QtXml>
#include <QMessageBox>
#include <QSharedMemory>
#include <QGraphicsScene>
#include "abuleduapplicationv1.h"
#include "abuleduconstantesv1.h"
#include "abuleduflatboutonv1.h"
#include "abuleduexercicewidgettelecommandev1.h"
#include "abuleduexercicewidgetairedetravailv1.h"
#include "abuleduexercicewidgetmessagev1.h"
#include "abuleduexercicecommonv1.h"
#include "abuledumessageboxv1.h"

/** @file abuleeduexercicev1.h
  * @version 1
  * @brief Cette classe permet de:
  * -gérer le moteur des exercices
  * -gérer les logs
  * Elle se présente sous la forme de widgets (attention, avant la modification du 20 novembre 2011 elle fournissait une QMainWindow).
  * Vous devez maintenant intégrer ces widgets dans votre fenêtre principale.
  * @link https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduExerciceV1
  */

class AbulEduExerciceV1: public QObject, public AbulEduExerciceCommonV1
{
    Q_OBJECT

public:

    /** Constructeur de la classe
      * @param QWidget *parent, un pointeur sur le widget parent de l'objet, initialise par defaut a 0
      */
    AbulEduExerciceV1(QWidget *parent = 0);

    /** Destructeur par defaut de la classe
      */
    ~AbulEduExerciceV1();

    //----------------- IHM --------------------- //
    /** l'IHM arrive dans la V1 de cette classe */

    /** Configure le niveau de l'exercice */
    virtual void setAbeLevel(QString s);
    /** Renseigne le nombre total de questions */
    virtual void setAbeNbTotalQuestions(int n);

    /** Affecte le theme du personnage dans la telecommande */
    virtual void setAbeThemeTeteTelecommande(QString s);
    /** Affecte la tete du personnage dans la telecommande */
    virtual void setAbeTeteTelecommande(QString s);
    /** Affecte une tete de personnage dans la telecommande en fonction du rapport entre les deux parametres */
    virtual qreal setAbeTeteForResult(int errors, int total);
    /** Affecte le theme du niveau (level) dans la telecommande */
    virtual void setAbeThemeLevelTelecommande(QString s);
    /** Affecte l'image du niveau dans la telecommande via setAbeLevel()*/

    //Les deux get pour pouvoir manipuler les objets depuis la state machine
    AbulEduExerciceWidgetTelecommandeV1  * getAbeExerciceTelecommandeV1();
    AbulEduExerciceWidgetAireDeTravailV1 * getAbeExerciceAireDeTravailV1();
    AbulEduExerciceWidgetMessageV1*        getAbeExerciceMessageV1();

public slots:
    /** Permet de connecter un signal (par exemple de la mainwindow) pour fermer l'exercice en cours */
    void slotAbeFermerExercice();

protected:

    /** Structure permettant de stocker les réponses des utilisateurs et la réponse attendue
      * @attention on met la réponse de l'utilisateur en premier, la réponse attendue en second
      */
    QList<QPair<QVariant,QVariant> > m_answers;

    /** Méthode permettant un affichage à l'utilisateur quand un module donné ne contient pas un type d'exercice */
    virtual void displayErrorMissingExercise(QString nomExercice);

private:
    /** Passage du module en mode debug ou pas
      * @brief m_localDebug, bool
      */
    bool m_localDebug;
    QString                                 m_themeTetes;       //Le theme des tete de la telecommande (asma/suzu/will/pingu/autre)
    QString                                 m_themeLevel;     //Le theme des niveaux de la telecommande (asma/suzu/will/pingu/autre)
    AbulEduExerciceWidgetTelecommandeV1*    m_uiAbulEduTelecommande;
    AbulEduExerciceWidgetAireDeTravailV1*   m_uiAbulEduAireDeTravail;
    AbulEduExerciceWidgetMessageV1*         m_uiAbulEduMessage;
};

#endif
