/**
  * Classe
  * @author 2011 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2012 Icham Sirat <icham.sirat@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distrAireDeTravailibuted in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "abuleduexercicewidgetmessagev1.h"
#include "ui_abuleduexercicewidgetmessagev1.h"

AbulEduExerciceWidgetMessageV1::AbulEduExerciceWidgetMessageV1(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduExerciceWidgetMessageV1)
{
    m_localDebug = false;
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    m_isDisplayError = true;
    m_hasImageFond = true;
    m_parent = parent;
    ui->setupUi(this);
    connect(parent,SIGNAL(dimensionsChangees()),this, SLOT(setDimensionsWidget()));
    setParent(parent);
    setStyleSheet("background-color: transparent;");
    setObjectName("message");
    ui->gvPrincipale->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    ui->gvPrincipale->setFrameShape(QFrame::NoFrame);
    gsPrincipale = new QGraphicsScene(ui->gvPrincipale);
    gsPrincipale->setObjectName("gsPrincipale");
    ui->gvPrincipale->setScene(gsPrincipale);
    ui->gvPrincipale->setSceneRect(0,0,ui->gvPrincipale->width(),ui->gvPrincipale->height());
    ui->gvPrincipale->setStyleSheet("background-color: transparent;");

    m_fond = new QGraphicsPixmapItem(); // L'image de fond
    gsPrincipale->addItem(m_fond);
    m_fond->setZValue(-10);

    frPrincipale = new QFrame(); // QFrame hébergeant les widgets de texte et autres
    frPrincipale->setStyleSheet("background-color: transparent;");
    frPrincipale->setObjectName(QString::fromUtf8("frPrincipale"));
    frPrincipale->setFrameShape(QFrame::NoFrame);

    m_vLayout = new QVBoxLayout(frPrincipale);
    m_vLayout->setSpacing(0);
    m_vLayout->setObjectName(QString::fromUtf8("m_vLayout"));
    m_vLayout->setContentsMargins(0, 0, 0, 0);

    lbTitre = new QLabel(frPrincipale); // Le titre de la fenêtre de consigne ou de bilan
    lbTitre->setObjectName(QString::fromUtf8("lbTitre"));
    lbTitre->setAlignment(Qt::AlignCenter);
    lbTitre->setFont(QFont(abeApp->font().family(),18,75));
    lbTitre->setText(trUtf8("Mon Joli Titre"));
    lbTitre->setTextInteractionFlags(Qt::NoTextInteraction);
    m_vLayout->addWidget(lbTitre);

    lbSeparationTitre = new QLabel(frPrincipale); // Trait de séparation entre le titre et le corps de la consigne
    lbSeparationTitre->setObjectName(QString::fromUtf8("lbSeparationTitre"));
    QPixmap trait;
    trait.load(":/abuleduexercicev1/fonds/traitseparation");
    lbSeparationTitre->setPixmap(trait);
    m_vLayout->addWidget(lbSeparationTitre);

    teConsigne = new QTextEdit(frPrincipale); // Le texte de la consigne ou le message d'évaluation
    teConsigne->setObjectName(QString::fromUtf8("teConsigne"));
    teConsigne->setFrameShape(QFrame::NoFrame);
    teConsigne->setTextInteractionFlags(Qt::NoTextInteraction);
    m_vLayout->addWidget(teConsigne);

    lbSeparationConsigne = new QLabel(frPrincipale); // Trait de séparation sous le message de consigne
    lbSeparationConsigne->setObjectName(QString::fromUtf8("lbSeparationConsigne"));
    lbSeparationConsigne->setPixmap(trait);
    m_vLayout->addWidget(lbSeparationConsigne);

    teTextExercice = new QTextEdit(frPrincipale); // Le champ pour le texte de l'exercice
    teTextExercice->setObjectName(QString::fromUtf8("teTextExercice"));
    teTextExercice->setTextInteractionFlags(Qt::NoTextInteraction);
    m_vLayout->addWidget(teTextExercice);

    spacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding); // Pour tasser le tout en haut
    m_vLayout->addSpacerItem(spacer);

    gsPrincipale->addWidget(frPrincipale);

    m_previousRatio = abeApp->getAbeApplicationDecorRatio();
//    setDimensionsWidget(); // Supprime car il n'a pas encore de dimensions utilisables
}

AbulEduExerciceWidgetMessageV1::~AbulEduExerciceWidgetMessageV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    delete ui;
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageClose()
{
    // a faire ou pas ?
    //setAttribute(Qt::WA_DeleteOnClose);
    close();
}

void AbulEduExerciceWidgetMessageV1::changeEvent(QEvent *e)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageSetImageFond(QString chemin)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_imageFond.load(chemin);
    m_fond->setPixmap(m_imageFond);
    m_fond->setPos(0,0);
    setDimensionsWidget();
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageShowImageFond(bool trueFalse)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_hasImageFond = trueFalse;
    if (m_hasImageFond == true)
    {
        setStyleSheet("background-color: transparent;");
    }
    else
    {
        setStyleSheet("background-color: #F8FCF4");
    }
}

bool AbulEduExerciceWidgetMessageV1::abeWidgetMessageHasImageFond()
{
    return m_hasImageFond;
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageSetTitre(QString titre)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    lbTitre->setText(titre);
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageSetConsigne(QString consigne)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    teConsigne->setHtml(consigne);
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageSetTexteExercice(QString textExercice)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    teTextExercice->setHtml(textExercice);
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageSetZoneTexteVisible(bool yesNo)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    lbSeparationConsigne->setVisible(yesNo);
    teTextExercice->setVisible(yesNo);
    if (m_hasImageFond == true)
    {
        if (yesNo)
        {
            abeWidgetMessageSetImageFond(":/abuleduexercicev1/fonds/fondmessagegrand");
        }
        else
        {
            abeWidgetMessageSetImageFond(":/abuleduexercicev1/fonds/fondmessagepetit");

        }
    }
    setDimensionsWidget();
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageSetIsDisplayError()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_isDisplayError = true;
}

void AbulEduExerciceWidgetMessageV1::abeWidgetMessageResize()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    setDimensionsWidget();
}

void AbulEduExerciceWidgetMessageV1::setDimensionsWidget()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    float ratio = abeApp->getAbeApplicationDecorRatio();
    if(m_localDebug) {
        qDebug()  << "  " << __FUNCTION__ <<  "  ratio "<<ratio;
    }

    QPixmap fond = m_imageFond;
    int largeur = fond.width()*ratio;
    if (m_hasImageFond == false)
    {
        largeur = m_parent->width()*2/3;
    }
    if(m_localDebug) {
        qDebug()  << "  " << __FUNCTION__ << "  Taille image de fond : "<<largeur<<" x "<<fond.height()*ratio;
    }
    frPrincipale->setFixedWidth(largeur - 100*ratio);
    lbTitre->setFixedWidth(largeur - 110*ratio);
    lbSeparationTitre->setFixedWidth(largeur - 110*ratio);
    lbSeparationConsigne->setFixedWidth(largeur - 110*ratio);
    teConsigne->setFixedWidth(largeur - 110*ratio);
    teTextExercice->setFixedWidth(largeur - 110*ratio);
    teTextExercice->document()->setTextWidth(largeur - 110*ratio -30);

    // Calcul de la hauteur nécessaire (et suffisante)
    int hauteur = 100*ratio;
    if(m_localDebug) {
        qDebug()  <<  "  Hauteur initiale " << hauteur;
    }

    lbTitre->setFixedHeight(lbTitre->fontMetrics().height() +10);
    if(m_localDebug) {
        qDebug()  << "  Hauteur titre " << lbTitre->height();
    }
    lbSeparationTitre->setFixedHeight(lbSeparationTitre->pixmap()->height());
    if(m_localDebug) {
        qDebug()  << "  Hauteur separation titre " << lbSeparationTitre->height();
    }
    if (m_previousRatio != 0.0 && m_previousRatio !=ratio)
    {
        if (teConsigne->toHtml().contains("img"))
        {
            teConsigne->setHtml(scaleImgWidthHtml(teConsigne->toHtml(),ratio/m_previousRatio));
            teConsigne->setHtml(scaleImgHeightHtml(teConsigne->toHtml(),ratio/m_previousRatio));
        }
        m_previousRatio = ratio;
    }
    teConsigne->setFixedHeight(teConsigne->document()->size().height());
    if(m_localDebug) {
        qDebug()  << "  Hauteur consigne " << teConsigne->height();
    }


    hauteur = hauteur
              + lbTitre->height()
              + lbSeparationTitre->height()
              + teConsigne->height();

    if(m_localDebug) {
        qDebug()  << "  Hauteur calcul1 " << hauteur;
    }


    lbSeparationConsigne->setFixedHeight(lbSeparationConsigne->pixmap()->height());

    if (m_hasImageFond == true)
    {
        teTextExercice->setFixedHeight(qMin((teTextExercice->document()->size().height()+10),qreal(frPrincipale->height()/2)-30));
    }
    else
    {
        teTextExercice->setFixedHeight(qMin((teTextExercice->document()->size().height()+10),qreal(m_parent->height()*0.5)));
    }

    if(teTextExercice->isVisible())
    {
        hauteur = hauteur
                  + lbSeparationConsigne->height()
                  + teTextExercice->height();
        if(m_localDebug) {
            qDebug()  << "  Hauteur separation consigne  " << lbSeparationTitre->height();
            qDebug()  << "  Hauteur texte exercice " << teTextExercice->height();
            qDebug()  << "  Hauteur calcul2 " << hauteur;
        }
    }

    fond = fond.scaled(largeur,hauteur, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    if(m_localDebug) {
        qDebug()  << "  Image de fond : " << largeur << " x " << hauteur;
    }
    m_fond->setPixmap(fond);
    m_fond->setPos(0,0);
    if (m_hasImageFond == true)
    {
        ui->gvPrincipale->setFixedSize(largeur, hauteur);
        frPrincipale->setFixedHeight(hauteur);
        frPrincipale->move(50*ratio, 50*ratio);
        this->setFixedWidth(largeur);
        this->setFixedHeight(hauteur);
    }
    else
    {
        ui->gvPrincipale->setFixedSize(largeur, m_parent->height()*0.85);
        frPrincipale->setFixedHeight(m_parent->height()*0.85);
        frPrincipale->move(50*ratio, 50*ratio);
        this->setFixedWidth(largeur);
        this->setFixedHeight(m_parent->height()*0.9);
    }
}

void AbulEduExerciceWidgetMessageV1::keyPressEvent(QKeyEvent *event)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QWidget::keyPressEvent(event);
}

void AbulEduExerciceWidgetMessageV1::keyReleaseEvent(QKeyEvent *event)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if (m_isDisplayError)
    {
        hide();
    }
    else
    {
        QWidget::keyReleaseEvent(event);
    }
}

void AbulEduExerciceWidgetMessageV1::mouseReleaseEvent(QMouseEvent* event)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if (m_isDisplayError == true)
    {
        hide();
        m_isDisplayError = false;
    }
    else
    {
        QWidget::mouseReleaseEvent(event);
    }
}

QString AbulEduExerciceWidgetMessageV1::scaleImgWidthHtml(QString html, float ratio)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    int imgOpenTag = html.indexOf("<img");
    int imgCloseTag = html.indexOf(">",imgOpenTag);
    QString contentTag = html.mid(imgOpenTag, imgCloseTag - imgOpenTag +1);
    QString previousContentTag = contentTag;
    if(m_localDebug) {
        qDebug() << "  Balise image : " << contentTag;
    }
    int keyWidth = contentTag.indexOf("width=");
    if (keyWidth == -1)
    {
        int keySrc = contentTag.indexOf("src=");
        int beginSrc = contentTag.indexOf("\"",keySrc);
        int endSrc = contentTag.indexOf("\"",beginSrc+1);
        QString contentKeySrc = contentTag.mid(beginSrc+1,endSrc-beginSrc-1);
        QPixmap copiePourTaille(contentKeySrc);
        contentTag.insert(endSrc+1," width=\""+QString::number(copiePourTaille.width()*ratio)+"\"");
        html.replace(previousContentTag,contentTag);
    }
    else
    {
        int beginWidth = contentTag.indexOf("\"",keyWidth);
        int endWidth = contentTag.indexOf("\"",beginWidth+1);
        QString contentKeyWidth = contentTag.mid(beginWidth+1,endWidth-beginWidth-1);
        bool widthConversion;
        float readWidth = contentKeyWidth.toFloat(&widthConversion);
        contentTag.replace(contentKeyWidth,QString::number(readWidth*ratio));
        if(m_localDebug) {
            qDebug()  << "Balise image modifiee : " << contentTag;
        }
        html.replace(previousContentTag,contentTag);
    }
    if(m_localDebug) {
        qDebug() << "  Message modifie" << html;
    }
    return html;
}

QString AbulEduExerciceWidgetMessageV1::scaleImgHeightHtml(QString html, float ratio)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    int imgOpenTag = html.indexOf("<img");
    int imgCloseTag = html.indexOf(">",imgOpenTag);
    QString contentTag = html.mid(imgOpenTag, imgCloseTag - imgOpenTag +1);
    QString previousContentTag = contentTag;
    if(m_localDebug) {
        qDebug()  << "Balise image : " << contentTag;
    }
    int keyHeight = contentTag.indexOf("height=");
    if (keyHeight == -1)
    {
        int keySrc = contentTag.indexOf("src=");
        int beginSrc = contentTag.indexOf("\"",keySrc);
        int endSrc = contentTag.indexOf("\"",beginSrc+1);
        QString contentKeySrc = contentTag.mid(beginSrc+1,endSrc-beginSrc-1);
        QPixmap copiePourTaille(contentKeySrc);
        contentTag.insert(endSrc+1," height=\""+QString::number(copiePourTaille.height()*ratio)+"\"");
        html.replace(previousContentTag,contentTag);
    }
    else
    {
        int beginHeight = contentTag.indexOf("\"",keyHeight);
        int endHeight = contentTag.indexOf("\"",beginHeight+1);
        QString contentKeyHeight = contentTag.mid(beginHeight+1,endHeight-beginHeight-1);
        bool heightConversion;
        float readHeight = contentKeyHeight.toFloat(&heightConversion);
        contentTag.replace(contentKeyHeight,QString::number(readHeight*ratio));
        if(m_localDebug) {
            qDebug()  << "Balise image modifiee : " << contentTag;
        }
        html.replace(previousContentTag,contentTag);
    }
    if(m_localDebug) {
        qDebug()  << "  Message modifie " <<html;
    }
    return html;
}

void AbulEduExerciceWidgetMessageV1::btnFermer_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    this->hide();
}
