/** Classe AbulEduBoxFileManagerV1
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  */

/*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduboxfilemanagerv1.h"
#include "ui_abuleduboxfilemanagerv1.h"

AbulEduBoxFileManagerV1::AbulEduBoxFileManagerV1(QWidget *parent, AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerOpenOrSave openOrSave) :
    QWidget(parent),
    ui(new Ui::AbulEduBoxFileManagerV1)
{
    m_localDebug = DEBUG_ABULEDUBOXFILEMANAGERV1;
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //Surtout pas sinon les logiciels "plantent": ils font un new dans le constructeur mais ensuite juste des show ... et nous dans le close
    //on se suicide ... donc le 2° show -> plantage
    //    setAttribute(Qt::WA_DeleteOnClose);

    if(parent == 0) {
        //Surtout pas modale sinon la popup sso ne peut pas se placer au dessus !
        //-> si mais il faut alors qu'elle soit aussi applicationmodale

        //2013.01.16 : decision pour la tablette
#ifdef __ABULEDUTABLETTEV1__MODE__
        setWindowModality(Qt::ApplicationModal);
#endif

    }
    ui->setupUi(this);
    this->setObjectName("AbulEduBoxFileManagerV1");
    m_sender = 0;

    //On ajoute un bandeau en haut avec le degradé et on l'intercalle pour qu'il soit visible mais pas au dessus du texte
    m_frmBandeauBackground = new QFrame(this);
    m_frmBandeauBackground->move(0,0);
    m_frmBandeauBackground->stackUnder(ui->stackedWidgetAbulEduBoxFileManager);
    m_frmBandeauBackground->setMinimumHeight(20);
    m_frmBandeauBackground->setMaximumHeight(20);
    m_frmBandeauBackground->setMinimumWidth(width());
    m_frmBandeauBackground->setMaximumWidth(5000);
    m_frmBandeauBackground->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    m_frmBandeauBackground->setStyleSheet(QString("QFrame {background-color: qlineargradient( x1:0 y1:0, x2:1 y2:0, stop:0 #69cc5e, stop:1 #72a2e6);}"));

    abeSetOpenOrSaveEnum(openOrSave);
    m_abuleduFile = QSharedPointer<AbulEduFileV1>(new AbulEduFileV1, &QObject::deleteLater);
    m_abuleduFile->setObjectName("depuis abuleduboxfilemanager");

    m_parent = parent;
    m_quelleBox = abePC;

    m_nam = abeApp->getAbeNetworkAccessManager();
    m_rpcAbulEduBox = new MaiaXmlRpcClient(QUrl("https://files.abuledu.net/index.php"), this);

    //-----------------------------------------------------
    //On créé l'arborescence si elle n'existe pas ...
    QDir dirTemp(abeApp->getAbeHomepath() + "/leterrier/" + "/abebox/" + abeApp->applicationName());
    if(!dirTemp.exists()) {
        dirTemp.mkpath(dirTemp.absolutePath());
    }
    QDir dirTempThumbs(abeApp->getAbeHomepath() + "/leterrier/" + "/abebox/" + abeApp->applicationName() + "/.thumbnails");
    if(!dirTempThumbs.exists()) {
        dirTempThumbs.mkpath(dirTempThumbs.absolutePath());
    }

    //Par defaut on affiche un des deux
    ui->twComplet->show();
    ui->twComplet->horizontalHeader()->setResizeMode(3,QHeaderView::Stretch);

    //Et on cache le moteur de recherche du widget mediatheque: abuledufilebox apporte le sien
    ui->widgetMediathequeGet->hide();
    ui->widgetMediathequeGet->abeSearchVisible(false);
    ui->lblAchetezEspaceStockage->hide();
    ui->lblSimpleAchetezEspaceStockage->hide();

    //On passe le signal, quand le fichier est downloade
    connect(ui->widgetMediathequeGet, SIGNAL(signalMediathequeFileDownloaded(int)), this, SIGNAL(signalAbeFileSelected()), Qt::UniqueConnection);

    m_dirABE           = QDir(abeApp->getAbeHomepath() + "/leterrier/abebox/" + abeApp->applicationName());
    m_dirThumbnailsABE = QDir(abeApp->getAbeHomepath() + "/leterrier/abebox/" + abeApp->applicationName() + "/.thumbnails");

    //On force le clic sur le bouton simple: c'est l'affichage par défaut
    //Astuce de barbu necessaire pour que le widget prenne a peu pres la bonne taille
    ui->btnSimple->click();
    ui->btnComplet->click();

    //Et par defaut on affiche le contenu local
    on_btnCompletMonPC_clicked();
    this->adjustSize();
    this->resize(minimumSize());
    //On centre la fenêtre
    QDesktopWidget *desktop = QApplication::desktop();
    QRect screenRect = desktop->availableGeometry(this);
    int desktop_width = screenRect.width();
    int desktop_height = screenRect.height();
    this->move((desktop_width-this->width())/2, (desktop_height-this->height())/2);


    /** @todo 2012 11 19 : desactive ce qui ne marche pas */
    ui->btnCompletAjouter->setEnabled(false);
    ui->btnCompletSupprimer->setEnabled(false);
    ui->btnCompletCopier->setEnabled(false);
    ui->btnCompletColler->setEnabled(false);
    //    ui->btnCompletMediatheque->setEnabled(false);

    if(parent != 0) {
        ui->btnCompletFermer->setEnabled(false);
        ui->btnSimpleFermer->setEnabled(false);
        //Super important si on veut que le widget garde son style (css) arrière plan et autre quand on l'intègre dans
        //un autre widget !
        setAttribute(Qt::WA_StyledBackground, true);
        if(m_localDebug) {
            qDebug() << "AbulEduBoxFileManagerV1: ********** " << objectName(); // << " ::::: " << styleSheet();
        }
        //        setStyleSheet( "AbulEduBoxFileManagerV1 {	background-color: qlineargradient( x1:0 y1:0, x2:0 y2:1, stop:0 #83b5f5, stop:1 #b8d2e5);}\n");
    }
    else {
        if(m_localDebug) {
            qDebug() << "AbulEduBoxFileManagerV1 sans parents : ********** " << objectName(); // << " ::::: " << styleSheet();
        }
    }
}

AbulEduBoxFileManagerV1::~AbulEduBoxFileManagerV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    delete ui;
    //    m_abuleduFile.clear();
}

void AbulEduBoxFileManagerV1::abeSetFile(QSharedPointer<AbulEduFileV1> file)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_abuleduFile = file;
    ui->widgetMediathequeGet->abeSetFile(file);
}

QSharedPointer<AbulEduFileV1> AbulEduBoxFileManagerV1::abeGetFile()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_abuleduFile;
}

void AbulEduBoxFileManagerV1::abeSetOpenOrSaveEnum(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerOpenOrSave openOrSave)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_openOrSave = openOrSave;
    nettoyageComplet();
}

AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerOpenOrSave AbulEduBoxFileManagerV1::abeGetOpenOrSaveEnum() const
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_openOrSave;
}

QObject *AbulEduBoxFileManagerV1::abeGetSender()
{
    return m_sender;
}

void AbulEduBoxFileManagerV1::abeSetSender(QObject *s)
{
    m_sender = s;
}

void AbulEduBoxFileManagerV1::abeSetDisplaySimpleOrCompleteEnum(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerDisplaySimpleOrComplete simpleOrComplete)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_displaySimpleOrComplete = simpleOrComplete;
    if(simpleOrComplete == abeDisplaySimple) {
        on_btnSimple_clicked();
    }
    else {
        on_btnComplet_clicked();
    }
}

AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerDisplaySimpleOrComplete AbulEduBoxFileManagerV1::abeGetDisplaySimpleOrCompleteEnum() const
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_displaySimpleOrComplete;
}

void AbulEduBoxFileManagerV1::abeRefresh(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation location)
{
    if(location == abePC) {
        on_btnCompletMonPC_clicked();
    }
    if(location == abeBoxPerso) {
        on_btnCompletBoxPerso_clicked();
    }
    if(location == abeBoxShare) {
        on_btnCompletBoxPartagee_clicked();
    }
    if(location == abeMediatheque) {
        on_btnCompletMediatheque_clicked();
    }
}

void AbulEduBoxFileManagerV1::rpcAbulEduBoxFileList(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QStringList iconesPasEnCache;       //icones de fichiers de box perso
    QStringList iconesGroupePasEnCache; //icones de fichiers de box partagees

    if(m_localDebug > 1) {
        qDebug() << "  =============================";
        qDebug() << "  " << v.toMap().value("Listing").toString();
        //        qDebug() << "  " << v.toMap();
        qDebug() << "  =============================";
    }
    QApplication::restoreOverrideCursor();

    //Structure retournée: $listingD .= "d;$lataille;$ladate;$entry;$groupe;\n";
    //ou                   $listingF .= "-;$lataille;$ladate;$entry;$groupe;\n";
    QStringList files = v.toMap().value("Listing").toString().split("\n");
    QStringList lesGroupes;

    //on est obligé d'avoir un 2° index pour eviter d'afficher des lignes vides quand on a les groupes
    int j = 0;
    for(int i = 0; i < files.count() ; i++) {
        QStringList infos = files.at(i).split(";");
        //Si on a un "groupe" on n'affiche rien, on ajoute dans la boite deroulante la valeur et c'est tout
        if(infos.at(0) == "g") {
            QString leGroupe = infos.at(4).trimmed();
            if(!lesGroupes.contains(leGroupe)) {
                lesGroupes << leGroupe;
            }
            continue;
        }
        if(infos.count() > 2) {
            ui->twComplet->insertRow(j);
            QTableWidgetItem *item1 = new QTableWidgetItem(infos.at(0));
            QTableWidgetItem *item2 = new QTableWidgetItem(infos.at(1));
            QTableWidgetItem *item3 = new QTableWidgetItem(infos.at(2));
            QTableWidgetItem *item4 = new QTableWidgetItem(infos.at(3));
            QTableWidgetItem *item5 = new QTableWidgetItem(infos.at(4));

            //Et on ajoute aussi dans la partie Simple
            QListWidgetItem *itemSimple = new QListWidgetItem();

            QString fileName = infos.at(3);
            QString leGroupe = infos.at(4).trimmed();
            if(!lesGroupes.contains(leGroupe)) {
                lesGroupes << leGroupe;
            }
            //On cherche une icone
            QFileInfo fi;
            //Le fichier cache local du thumbnail est prefixe du nom de la box
            fi.setFile(m_dirThumbnailsABE.absolutePath() + "/" + ABE_ENUMTOSTRING().value(m_quelleBox) + "-" + fileName.replace(".abe",".png"));
            if(!fi.exists()) {
                if(infos.at(4) != "perso") {
                    iconesGroupePasEnCache << infos.at(3) + ":" + infos.at(4);
                }
                else {
                    iconesPasEnCache << infos.at(3);
                }
            }
            else {
                itemSimple->setIcon(QIcon(fi.absoluteFilePath())); // ajout de l'icone sur l'item
                item1->setIcon(QIcon(fi.absoluteFilePath()));
            }

            //Dans l'ancien temps on voulait avoir éventuellement des sous répertoires
            //            if(infos.at(0) == "d")
            //                item1->setIcon(QIcon(QPixmap(":/data/images/dossier-24.png")));
            //            else if(infos.at(0) == "-")
            //                item1->setIcon(QIcon(QPixmap(":/data/images/fichier-24.png")));

            fileName = infos.at(3);
            item1->setData(Qt::UserRole+1, fileName);
            item1->setData(Qt::UserRole+2, leGroupe);
            item2->setData(Qt::UserRole+1, fileName);
            item2->setData(Qt::UserRole+2, leGroupe);
            item3->setData(Qt::UserRole+1, fileName);
            item3->setData(Qt::UserRole+2, leGroupe);
            item4->setData(Qt::UserRole+1, fileName);
            item4->setData(Qt::UserRole+2, leGroupe);
            item5->setData(Qt::UserRole+1, fileName);
            item5->setData(Qt::UserRole+2, leGroupe);

            ui->twComplet->setItem(j, 0, item1);
            ui->twComplet->setItem(j, 1, item2);
            ui->twComplet->setItem(j, 2, item3);
            ui->twComplet->setItem(j, 3, item4);
            ui->twComplet->setItem(j, 4, item5);

            //            itemSimple->setIcon(QIcon("/usr/share/icons/autoscan-network.png")); // ajout de l'icone sur l'item
            itemSimple->setData(Qt::UserRole+1, fileName);
            itemSimple->setData(Qt::UserRole+2, leGroupe);
            ui->lwSimple->insertItem(j,itemSimple);
        }
        j++;
    }

    ui->cbGroupeSauvegarder->addItems(lesGroupes);

    if(iconesPasEnCache.size() > 0 ||  iconesGroupePasEnCache.size() > 0) {
        QVariantMap tableau;
        tableau["appname"]       = abeApp->applicationName();
        tableau["filelist"]      = iconesPasEnCache.join(";");
        tableau["fileGrouplist"] = iconesGroupePasEnCache.join(";");

        if (m_localDebug) qDebug() << ">> Icones pas en cache ... on cherche " << tableau;

        QApplication::setOverrideCursor(Qt::WaitCursor);

        QList<QVariant> params = QList<QVariant>() << tableau;
        m_rpcAbulEduBox->call("abuledu.files.getThumbnails", params,
                              this, SLOT(rpcGetThumbnails(QVariant&, QNetworkReply*)),
                              this, SLOT(rpcFault(int, const QString &)));
    }

    ui->twComplet->resizeColumnsToContents();
    ui->twComplet->horizontalHeader()->setResizeMode(3,QHeaderView::Stretch);
}

void AbulEduBoxFileManagerV1::rpcFault(int code, const QString &errorMessage)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  ===================================";
        qDebug() << "  " << code << errorMessage;
        qDebug() << "  ===================================";
    }

    m_rpcAbulEduBox->activateProgressBar(false);
    //Tentative de réouverture d'une session SSO en cas de timeout
    if(code == -32300) {
        //2013.01.16 : decision pour la tablette
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
        m_nam->abeSSOLogin();
#endif
    }
}

void AbulEduBoxFileManagerV1::rpcFaultFileUpload(int code, const QString &errorMessage)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  ===================================";
        qDebug() << "  " << code << errorMessage;
        qDebug() << "  ===================================";
    }

    //Tentative de réouverture d'une session SSO en cas de timeout
    if(code == -32300) {
        //2013.01.16 : decision pour la tablette
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
        m_nam->abeSSOLogin();
#endif
    }
    emit signalAbeFileSaved(abePC,m_abuleduFile->abeFileGetFileName().absoluteFilePath(),false);
}

void AbulEduBoxFileManagerV1::rpcFileUpload(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
      qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
      qDebug() << "  ============================================================";
      qDebug() << "  " << v.toMap();
      qDebug() << "  ============================================================";
    }

    /**
      * @todo gérer l'erreur éventuelle: le code retour du serveur (la transmission réseau est ptet ok mais c'est pas pour autant que le fichier est bien stocké "la bas"
      */
    //et on supprime le fichier temporaire
    emit signalAbeFileSaved(abeBoxPerso,m_abuleduFile->abeFileGetFileName().fileName(), true);

}

void AbulEduBoxFileManagerV1::slotUploadFile(QString fileName, QString groupe)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Si on est authentifie
    if(m_nam->abeSSOAuthenticationStatus() == 1) {
        QFileInfo fi(fileName);
        QCryptographicHash md5(QCryptographicHash::Md5);
        if(fi.exists()) {
            QString fichierBase64;
            QFile fic(fileName);
            if (fic.open(QIODevice::ReadOnly)) {
                QByteArray fileData = fic.readAll();
                fichierBase64 = fileData.toBase64();
                md5.addData(fileData);
                fic.close();
            }
            else {
                if (m_localDebug) qDebug() << "Erreur d'ouverture du fichier ";
                return;
            }

            QVariantMap tableau;
            tableau["appname"]      = abeApp->applicationName();
            tableau["fichier_nom"]  = fi.fileName();
            tableau["groupe_nom"]   = groupe;
            tableau["fichier_md5"]  = QString(md5.result().toHex());
            if (m_localDebug) {
                qDebug() << "on envoie " << tableau;
            }
            tableau["fichier_content"]  = fichierBase64;
            QList<QVariant> params = QList<QVariant>() << tableau;

            m_rpcAbulEduBox->activateProgressBar(true,true);
            m_rpcAbulEduBox->call("abuledu.files.setFile", params,
                                  this, SLOT(rpcFileUpload(QVariant&, QNetworkReply*)),
                                  this, SLOT(rpcFaultFileUpload(int, const QString &)));

        }
    }
    else {
        //2013.01.16 : decision pour la tablette
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
        m_nam->abeSSOLogin();
        m_nam->abeOnLoginSuccessGoto(this, SLOT(on_btnNomFichierSauvegarder_clicked()));
#endif
    }
}

void AbulEduBoxFileManagerV1::on_twComplet_itemDoubleClicked(QTableWidgetItem *item)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QString filename(item->data(Qt::UserRole+1).toString());
    if (m_localDebug) qDebug() << "AbulEduBoxFileManagerV1::on_twComplet_itemDoubleClicked " << filename;

    //Si on est en mode sauvegarde on demande confirmation pour écraser le fichier sélectionné
    if(m_openOrSave == abeSave) {
        sauvegardeFichier(filename);
    }
    else {
        on_btnCompletOuvrir_clicked();
    }
}

void AbulEduBoxFileManagerV1::rpcAbulEduBoxGetFile(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_localDebug > 1) {
        qDebug() << "  =============================";
        qDebug() << "  " << v;
        qDebug() << "  =============================";
    }
    QApplication::restoreOverrideCursor();
    m_rpcAbulEduBox->activateProgressBar(false);
    if(v.toMap().value("status").toString() == "200") {
        //Soit un fichier temporaire
        //QTemporaryFile *file = new QTemporaryFile(QDir::tempPath() + "/AbulEduMediathequeGetV1-XXXXXXX" + v.toMap().value("ID").toString() + ".abe");


        //Soit un fichier dont le nom est evident pour qu'on puisse ne pas le re-telecharger dans 20 secondes
        //si on veut le réouvrir / rechercher à nouveau
        //QFile file(QDir::tempPath() + "/AbulEduMediathequeGetV1-" + m_docID + ".abe");
        //ou son nom officiel sur la boutique
        QString temporaire(QDateTime::currentDateTime().toString("hhmmsszzz"));
        QString filename(QDir::tempPath() + "/" + v.toMap().value("fichier_name", "AbulEduBoxFileManagerV1-" + temporaire + ".abe").toString());
        QFile file(filename);

        if(file.open(QIODevice::WriteOnly)) {
            QByteArray temp = QByteArray::fromBase64(v.toMap().value("fichier_content").toByteArray());
            file.write(temp, temp.size());
            file.close();

            //A noter, c'est une mauvaise idee de passer par un qdatastream: il ajoute quelques octets en entete qui
            //plantent tout !
            //QDataStream out(&file);
            //out << QByteArray::fromBase64(v.toMap().value("fichier_content").toByteArray());
            //file.close();

            if (m_localDebug) qDebug() << "Fichier sauvegarde dans :" << file.fileName();

            //Si c'est un fichier abe on l'ouvre nous-même
            if(file.fileName().endsWith(".abe")) {
                //            m_abuleduFile = QSharedPointer<AbulEduFileV1>(new AbulEduFileV1, &QObject::deleteLater);
                //            m_abuleduFile->setObjectName("depuis abuleduboxfilemanager (2)");
                m_abuleduFile->abeFileOpen(filename);
                emit signalAbeFileSelected();
            }
            else {
                QDesktopServices::openUrl(QUrl(filename, QUrl::TolerantMode));
            }
        }
    }
    else {
        //Erreur, impossible de télécharger/écrire le fichier
        //        emit signalMediathequeFileDownloaded(-1);
        AbulEduMessageBoxV1 *box = new AbulEduMessageBoxV1(trUtf8("Erreur"), trUtf8("Erreur de téléchargement du fichier !"),this);
        box->show();
    }
}

void AbulEduBoxFileManagerV1::on_leRecherche_returnPressed()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->widgetMediathequeGet->abeClear(true);
    ui->widgetMediathequeGet->abeSearch(ui->leRecherche->text());
    ui->leRecherche->clear();
}

void AbulEduBoxFileManagerV1::on_btnSimple_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Affichage simple: on affiche les 5 derniers ouverts / les fichiers de démo / quoi ?
    //Pour l'instant on affiche tout le contenu du répertoire

    ui->stackedWidgetAbulEduBoxFileManager->setCurrentWidget(ui->pageSimple);


}

void AbulEduBoxFileManagerV1::on_btnComplet_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->stackedWidgetAbulEduBoxFileManager->setCurrentWidget(ui->pageComplet);
}

void AbulEduBoxFileManagerV1::on_lwSimple_itemDoubleClicked(QListWidgetItem *item)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QString filename(item->data(Qt::UserRole+1).toString());
    //Si on est en mode sauvegarde on demande confirmation pour écraser le fichier sélectionné

    if(m_openOrSave == abeSave) {
        sauvegardeFichier(filename);
    }
    else {
        on_btnSimpleOuvrir_clicked();
    }
}

void AbulEduBoxFileManagerV1::on_btnSimpleMonPC_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    on_btnCompletMonPC_clicked();
}

void AbulEduBoxFileManagerV1::on_btnSimpleBoxPerso_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_quelleBox = abeBoxPerso;
    nettoyageComplet();

    //Si on est authentifie
    if(m_nam->abeSSOAuthenticationStatus() == 1) {

        QVariantMap tableau;
        tableau["appname"]  = abeApp->applicationName();
        QList<QVariant> params = QList<QVariant>() << tableau;
        m_rpcAbulEduBox->call("abuledu.files.getList", params,
                              this, SLOT(rpcAbulEduBoxFileList(QVariant&, QNetworkReply*)),
                              this, SLOT(rpcFault(int, const QString &)));
    }
    else {
        //2013.01.16 : decision pour la tablette
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
        m_nam->abeSSOLogin();
        m_nam->abeOnLoginSuccessGoto(this, SLOT(on_btnSimpleBoxPerso_clicked()));
        m_nam->abeOnLoginFailureGoto(this, SLOT(ifAuthenticateFailed()));
#endif

    }

}

void AbulEduBoxFileManagerV1::on_btnSimpleBoxPartagee_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    on_btnCompletBoxPartagee_clicked();
}

void AbulEduBoxFileManagerV1::on_btnSimpleOuvrir_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->lwSimple->selectedItems().count() > 0) {
        QString filename(ui->lwSimple->selectedItems().at(0)->data(Qt::UserRole+1).toString());
        QString groupe(ui->lwSimple->selectedItems().at(0)->data(Qt::UserRole+2).toString());
        ouvertureFichier(filename,groupe);
    }
}

void AbulEduBoxFileManagerV1::on_btnCompletMediatheque_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_quelleBox = abeMediatheque;

    nettoyageComplet();

    //Si on est authentifie
    if(m_nam->abeSSOAuthenticationStatus() == 1) {
        ui->twComplet->hide();

        ui->widgetMediathequeGet->show();

        //Sur la mediatheque on ne peut pas publier / coller / supprimer des ressources ...
        ui->btnCompletAjouter->setEnabled(false);
        ui->btnCompletSupprimer->setEnabled(false);
        ui->btnCompletColler->setEnabled(false);

        //Et par défaut on affiche les X premiers documents disponibles sur la médiathèque
        ui->leRecherche->setText("*");
        on_leRecherche_returnPressed();
    }
    else {
        //2013.01.16 : decision pour la tablette
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
        m_nam->abeSSOLogin();
        m_nam->abeOnLoginSuccessGoto(this, SLOT(on_btnCompletMediatheque_clicked()));
        m_nam->abeOnLoginFailureGoto(this, SLOT(ifAuthenticateFailed()));
#endif

    }



}

void AbulEduBoxFileManagerV1::on_btnCompletBoxPerso_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    on_btnSimpleBoxPerso_clicked();
}

void AbulEduBoxFileManagerV1::on_btnCompletBoxPartagee_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_quelleBox = abeBoxShare;
    nettoyageComplet();

    ui->lblAchetezEspaceStockage->show();
    ui->lblSimpleAchetezEspaceStockage->show();
    //Si on est authentifie
    if(m_nam->abeSSOAuthenticationStatus() == 1) {

        QVariantMap tableau;
        tableau["appname"]  = abeApp->applicationName();
        QList<QVariant> params = QList<QVariant>() << tableau;
        m_rpcAbulEduBox->call("abuledu.files.getSharedList", params,
                              this, SLOT(rpcAbulEduBoxFileList(QVariant&, QNetworkReply*)),
                              this, SLOT(rpcFault(int, const QString &)));
    }
    else {
        m_nam->abeSSOLogin();
        m_nam->abeOnLoginSuccessGoto(this, SLOT(on_btnCompletBoxPerso_clicked()));
        m_nam->abeOnLoginFailureGoto(this, SLOT(ifAuthenticateFailed()));
    }
}

void AbulEduBoxFileManagerV1::on_btnCompletOuvrir_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->twComplet->selectedItems().count() > 0) {
        QString filename(ui->twComplet->selectedItems().at(0)->data(Qt::UserRole+1).toString());
        QString groupe(ui->twComplet->selectedItems().at(0)->data(Qt::UserRole+2).toString());
        if (m_localDebug) {
            qDebug() << "AbulEduBoxFileManagerV1::on_btnCompletOuvrir_clicked " << filename << " du groupe " << groupe;
        }
        ouvertureFichier(filename,groupe);
    }
}

void AbulEduBoxFileManagerV1::on_btnCompletCopier_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

}

void AbulEduBoxFileManagerV1::on_btnCompletColler_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

}

void AbulEduBoxFileManagerV1::on_btnCompletSupprimer_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

}

void AbulEduBoxFileManagerV1::on_btnCompletAjouter_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->twComplet->selectedItems().count() > 0) {
        if (m_localDebug) qDebug() << "On est sur la ligne " << ui->twComplet->selectedItems().at(0)->row();
        for(int i = 0; i < ui->twComplet->selectedItems().count(); i++) {
            QFileInfo fi(ui->twComplet->selectedItems().at(i)->data(Qt::UserRole+1).toString());
            if (m_localDebug) qDebug() << "Fichier : " << fi.absoluteFilePath();
            if(fi.exists()) {
                int ret = QMessageBox::question(0,trUtf8("Supprimer un fichier"), trUtf8("Êtes vous sûr de vouloir supprimer ce fichier ?"), QMessageBox::Yes | QMessageBox::No);
                if (m_localDebug) qDebug() << "Réponse : " << ret;
                if(ret == QMessageBox::Yes) {
                    QFile::remove(ui->twComplet->selectedItems().at(i)->data(Qt::UserRole+1).toString());
                    delete ui->twComplet->selectedItems().at(i);
                    ui->twComplet->removeRow(ui->twComplet->selectedItems().at(0)->row());
                    return;
                }
                else {
                    return;
                }
            }
        }
    }

}

void AbulEduBoxFileManagerV1::nettoyageComplet()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    bool enableAll;

    //Si on a un parent (on est intégré dans un autre widget) on cache le bouton fermer
    if(parent() != 0) {
        ui->btnCompletFermer->setEnabled(false);
        ui->btnSimpleFermer->setEnabled(false);
    }

    ui->twComplet->show();
    ui->twComplet->clearContents();
    ui->twComplet->setRowCount(0);
    ui->widgetMediathequeGet->hide();
    ui->twComplet->horizontalHeader()->setResizeMode(3,QHeaderView::Stretch);
    ui->lblAchetezEspaceStockage->hide();
    ui->lblSimpleAchetezEspaceStockage->hide();

    ui->lwSimple->clear();
    ui->cbGroupeSauvegarder->clear();

    if(m_openOrSave == abeSave) {
        //On affiche le choix du groupe
        //        ui->lblNomGroupeSauvegarder->show();
        //        ui->cbGroupeSauvegarder->show();

        //Le choix du groupe n'est possible que sur la box partagée
        if(m_quelleBox == abeBoxShare) {
            ui->lblNomGroupeSauvegarder->setEnabled(true);
            ui->cbGroupeSauvegarder->setEnabled(true);
        }
        else {
            ui->lblNomGroupeSauvegarder->setEnabled(false);
            ui->cbGroupeSauvegarder->setEnabled(false);
        }
        ui->widgetFichierSauvegarder->show();
        enableAll = false;
        //On cache la possibilité de passer en affichage simple
        ui->btnSimple->hide();
        //Et le moteur de recherche qui n'est pas utile
        ui->leRecherche->hide();
        //Et les boutons inutiles
        ui->btnCompletCopier->hide();
        ui->btnCompletColler->hide();
        ui->btnCompletAjouter->hide();
        ui->btnCompletSupprimer->hide();
        ui->btnCompletOuvrir->hide();
        setWindowTitle(trUtf8("Gestionnaire de fichiers AbulÉdu : Mode Sauvegarde"));
    }
    else {
        //        ui->lblNomGroupeSauvegarder->hide();
        //        ui->cbGroupeSauvegarder->hide();
        ui->widgetFichierSauvegarder->hide();
        enableAll = true;
        setWindowTitle(trUtf8("Gestionnaire de fichiers AbulÉdu : Mode Ouverture"));
    }
    //On cache la mediatheque quand on est en mode save
    ui->btnCompletMediatheque->setVisible(enableAll);

    //Ouvrir un fichier quand on est en mode save ...
    ui->btnSimpleOuvrir->setEnabled(enableAll);
    ui->btnCompletOuvrir->setEnabled(enableAll);

    //Mise en surbrillance de la box actuellement selectionnée
    if(m_quelleBox == abePC) {
        ui->btnCompletMonPC->setChecked(true);
        ui->btnSimpleMonPC->setChecked(true);
    }
    if(m_quelleBox == abeBoxPerso) {
        ui->btnCompletBoxPerso->setChecked(true);
        ui->btnSimpleBoxPerso->setChecked(true);
    }
    if(m_quelleBox == abeBoxShare) {
        ui->btnCompletBoxPartagee->setChecked(true);
        ui->btnSimpleBoxPartagee->setChecked(true);
    }
    if(m_quelleBox == abeMediatheque) {
        ui->btnCompletMediatheque->setChecked(true);
    }

    /** @todo 2012 11 19 : desactive ce qui ne marche pas */
    //    ui->leRecherche->setEnabled(enableAll);
    //    ui->btnCompletCopier->setEnabled(enableAll);
    //    ui->btnCompletColler->setEnabled(enableAll);
    //    ui->btnCompletAjouter->setEnabled(enableAll);
    //    ui->btnCompletSupprimer->setEnabled(enableAll);
}

void AbulEduBoxFileManagerV1::on_btnCompletMonPC_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_quelleBox = abePC;
    nettoyageComplet();

    ui->btnCompletAjouter->setEnabled(false);

    QStringList filters;
    filters << "*.abe";

    m_dirABE.setNameFilters(filters);
    m_dirABE.setFilter(QDir::Files);
    m_dirABE.setSorting(QDir::Name | QDir::IgnoreCase | QDir::LocaleAware);
    QFileInfoList list = m_dirABE.entryInfoList();
    for(int i = 0; i < list.count(); i++)
    {
        // ========================== SIMPLE DEBUT
        if (m_localDebug) qDebug() << "  " << __FUNCTION__<< " Fichier dans rep utilisateur: " << list.at(i).completeBaseName();
        //Pour tests on affiche toutes les vignettes d'un répertoire
        QListWidgetItem *item = new QListWidgetItem();

        QFileInfo ficone(m_dirThumbnailsABE.absolutePath() + "/" + list.at(i).completeBaseName() + ".png");
        //Si l'icone n'existe pas on demande a l'extraire ...
        if(!ficone.exists()) {
            AbulEduFileV1::abeFileExtractIcon(list.at(i).absoluteFilePath(),ficone.absoluteFilePath());
        }

        QIcon icone(ficone.absoluteFilePath());//pour la mettre  à coté de l'item
        item->setIcon(icone); // ajout de la petite icone sur l'item
        //        item->setText(list.at(i).fileName());
        item->setData(Qt::UserRole+1,list.at(i).absoluteFilePath());
        //        item->setCheckState(Qt::Unchecked);
        ui->lwSimple->insertItem(i,item);
        // ========================== SIMPLE FIN


        // ========================== COMPLET DEBUT
        if (m_localDebug) qDebug() << "  " << __FUNCTION__ << " Fichier dans rep utilisateur (complet): " << list.at(i).completeBaseName();
        ui->twComplet->insertRow(i);

        QTableWidgetItem *item1 = new QTableWidgetItem();
        item1->setIcon(icone);
        QTableWidgetItem *item2 = new QTableWidgetItem(QString::number(list.at(i).size()));
        QTableWidgetItem *item3 = new QTableWidgetItem(list.at(i).lastModified().toString("dd/MM/yyyy h:m"));
        QTableWidgetItem *item4 = new QTableWidgetItem(list.at(i).fileName());

        item1->setData(Qt::UserRole+1, list.at(i).absoluteFilePath());
        item2->setData(Qt::UserRole+1, list.at(i).absoluteFilePath());
        item3->setData(Qt::UserRole+1, list.at(i).absoluteFilePath());
        item4->setData(Qt::UserRole+1, list.at(i).absoluteFilePath());

        ui->twComplet->setItem(i, 0, item1);
        ui->twComplet->setItem(i, 1, item2);
        ui->twComplet->setItem(i, 2, item3);
        ui->twComplet->setItem(i, 3, item4);
        // ========================== COMPLET FIN
    }

    //Et dans le répertoire de l'application
    QDir dirAppABE;
#ifdef Q_WS_MAC
    dirAppABE.setPath(QCoreApplication::applicationDirPath() + "/data/");
#else
    dirAppABE.setPath(QDir::currentPath() + "/data/");
#endif
    dirAppABE.setNameFilters(filters);
    dirAppABE.setFilter(QDir::Files);
    dirAppABE.setSorting(QDir::Name | QDir::IgnoreCase | QDir::LocaleAware);
    list = dirAppABE.entryInfoList();
    for(int i = 0; i < list.count(); i++)
    {
        // ========================== SIMPLE DEBUT
        if (m_localDebug) qDebug() << "  " << __FUNCTION__<< " Fichier dans rep appli: " << list.at(i).completeBaseName();

        //Pour tests on affiche toutes les vignettes d'un répertoire
        QListWidgetItem *item = new QListWidgetItem();

        QFileInfo ficone(m_dirThumbnailsABE.absolutePath() + "/" + list.at(i).completeBaseName() + ".png");
        //Si l'icone n'existe pas on demande a l'extraire ...
        if(!ficone.exists()) {
            AbulEduFileV1::abeFileExtractIcon(list.at(i).absoluteFilePath(),ficone.absoluteFilePath());
        }

        QIcon icone(ficone.absoluteFilePath());//pour la mettre  à coté de l'item
        item->setIcon(icone); // ajout de la petite icone sur l'item
        //        item->setText(list.at(i).fileName());
        item->setData(Qt::UserRole+1,list.at(i).absoluteFilePath());
        //        item->setCheckState(Qt::Unchecked);
        ui->lwSimple->insertItem(i,item);
        // ========================== SIMPLE FIN


        // ========================== COMPLET DEBUT
        if (m_localDebug) qDebug() << "  " << __FUNCTION__ << " Fichier dans rep de l'appli (complet): " << list.at(i).completeBaseName();
        ui->twComplet->insertRow(i);

        QTableWidgetItem *item1 = new QTableWidgetItem();
        item1->setIcon(icone);
        QTableWidgetItem *item2 = new QTableWidgetItem(QString::number(list.at(i).size()));
        QTableWidgetItem *item3 = new QTableWidgetItem(list.at(i).lastModified().toString("dd/MM/yyyy h:m"));
        QTableWidgetItem *item4 = new QTableWidgetItem(list.at(i).fileName());

        item1->setData(Qt::UserRole+1, list.at(i).absoluteFilePath());
        item2->setData(Qt::UserRole+1, list.at(i).absoluteFilePath());
        item3->setData(Qt::UserRole+1, list.at(i).absoluteFilePath());
        item4->setData(Qt::UserRole+1, list.at(i).absoluteFilePath());

        ui->twComplet->setItem(i, 0, item1);
        ui->twComplet->setItem(i, 1, item2);
        ui->twComplet->setItem(i, 2, item3);
        ui->twComplet->setItem(i, 3, item4);
        // ========================== COMPLET FIN

    }

}

void AbulEduBoxFileManagerV1::on_btnNomFichierSauvegarder_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QDir *m_dir = new QDir(abeApp->getAbeHomepath() + "/leterrier/" + "/abebox/" + abeApp->applicationName());
    if(m_quelleBox == abePC)
    {
        bool codeRetour = m_abuleduFile->abeFileExportFile(m_dir->absolutePath() + "/" + ui->leNomFichierSauvegarder->text().trimmed());
        emit signalAbeFileSaved(abePC,m_abuleduFile->abeFileGetFileName().absoluteFilePath(),codeRetour);
        if (m_localDebug) qDebug()<<"signal émis par "<<objectName()<<" avec PC et "<<m_abuleduFile->abeFileGetFileName().absoluteFilePath();
    }
    else if(m_quelleBox == abeBoxPerso)
    {
        //on exporte dans un fichier temporaire
        bool codeRetour = m_abuleduFile->abeFileExportFile(QDir::tempPath() + "/" + ui->leNomFichierSauvegarder->text().trimmed());
        if(codeRetour) {
            //et ensuite on uploade
            slotUploadFile(m_abuleduFile->abeFileGetFileName().absoluteFilePath());
            if (m_localDebug) qDebug()<<"signal émis "<<objectName()<<" avec BoxPerso et "<<m_abuleduFile->abeFileGetFileName().fileName();
        }
        else {
            emit signalAbeFileSaved(abePC,m_abuleduFile->abeFileGetFileName().absoluteFilePath(),codeRetour);
        }
    }
    else if(m_quelleBox == abeBoxShare)
    {
        //on exporte dans un fichier temporaire
        bool codeRetour = m_abuleduFile->abeFileExportFile(QDir::tempPath() + "/" + ui->leNomFichierSauvegarder->text().trimmed());
        if(codeRetour) {
            //et ensuite on uploade
            slotUploadFile(m_abuleduFile->abeFileGetFileName().absoluteFilePath(), ui->cbGroupeSauvegarder->currentText().trimmed());
            if (m_localDebug) qDebug()<<"signal émis "<<objectName()<<" avec BoxPerso et "<<m_abuleduFile->abeFileGetFileName().fileName();
        }
        else {
            emit signalAbeFileSaved(abePC,m_abuleduFile->abeFileGetFileName().absoluteFilePath(),codeRetour);
        }
    }
    else {
        AbulEduMessageBoxV1 *box = new AbulEduMessageBoxV1(trUtf8("Erreur"), trUtf8("Erreur de destination de sauvegarde !"));
        box->show();
    }

    if (m_localDebug) qDebug()<<"sauvegarde du fichier .abe dans le dossier "<<m_abuleduFile->abeFileGetFileName().absoluteFilePath();
    if (m_parent)
    {
        hide();
    }
    else
    {
        close();
    }
    emit signalAbeFileCloseOrHide();
}

void AbulEduBoxFileManagerV1::rpcGetThumbnails(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_localDebug > 1) {
        qDebug() << "  =============================";
        qDebug() << "  " << v;
        qDebug() << "  =============================";
    }
    QApplication::restoreOverrideCursor();

    //QVariant(QVariantMap,
    //    QMap(
    //          ("Message", QVariant(QString, "") )
    //          ("SendOK" ,  QVariant(bool, true) )
    //          ( "status" ,  QVariant(QString, "200") )
    //          ( "tabThumb" ,  QVariant(
    //               QVariantList, (QVariant(QVariantMap, QMap(
    //                      ("name", QVariant(QString, "les trois frères.abe") )
    //                      ( "thumb" ,  QVariant(QString,
    if(v.toMap().value("status").toString() == "200") {
        for(int i = 0; i < v.toMap().value("tabThumb").toList().size(); i++) {
            QString nomFichier = v.toMap().value("tabThumb").toList().at(i).toMap().value("name").toString();

            QString nomFichierCacheLocal = m_dirThumbnailsABE.absolutePath() + "/" + ABE_ENUMTOSTRING().value(m_quelleBox) + "-" + nomFichier.replace(".abe",".png");
            QFile file(nomFichierCacheLocal);
            if(file.open(QIODevice::WriteOnly)) {
                QByteArray temp = QByteArray::fromBase64(v.toMap().value("tabThumb").toList().at(i).toMap().value("thumb").toByteArray());
                file.write(temp, temp.size());
                file.close();
            }
        }
    }
}

void AbulEduBoxFileManagerV1::on_twComplet_itemClicked(QTableWidgetItem *item)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Si on est en mode sauvegarde
    if(m_openOrSave == abeSave) {
        QFileInfo fi(item->data(Qt::UserRole+1).toString());
        if (m_localDebug) qDebug() << "AbulEduBoxFileManagerV1::on_twComplet_itemClicked " << fi.baseName();
        ui->leNomFichierSauvegarder->setText(fi.baseName());
        if (m_localDebug) qDebug() << " =========== verif =========" << ui->btnCompletAjouter->isEnabled() << " et " << ui->btnCompletColler->isEnabled();
    }
}

void AbulEduBoxFileManagerV1::sauvegardeFichier(QString filename)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //        AbulEduMessageBoxV1 *box = new AbulEduMessageBoxV1(trUtf8("Confirmation"), trUtf8("Êtes-vous certain de vouloir remplacer le fichier %1 ?").arg(fi.baseName()),this);
    //        box->show();
    QFileInfo fi(filename);
    int i = QMessageBox::question(this,
                                  trUtf8("Confirmation"),
                                  trUtf8("Êtes-vous certain de vouloir remplacer le fichier %1 ?").arg(fi.baseName()),
                                  QMessageBox::Yes | QMessageBox::No
                                  );

    if(i == QMessageBox::Yes) {
        ui->leNomFichierSauvegarder->setText(fi.baseName());
        on_btnNomFichierSauvegarder_clicked();
    }
    if (m_localDebug) qDebug() << "Retour : " << i;
}

void AbulEduBoxFileManagerV1::ouvertureFichier(QString filename, QString groupe)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_quelleBox == abePC) {
        //Fichier local
        m_abuleduFile->abeFileOpen(filename);
        emit signalAbeFileSelected();
    }
    else {
        //Téléchargement
        QVariantMap tableau;
        tableau["appname"]  = abeApp->applicationName();
        tableau["filename"] = filename;
        tableau["groupe"] = groupe;
        QList<QVariant> params = QList<QVariant>() << tableau;
        if (m_localDebug) qDebug() << "On telecharge : " << tableau;
        m_rpcAbulEduBox->activateProgressBar(true);
        m_rpcAbulEduBox->call("abuledu.files.getFile", params,
                              this, SLOT(rpcAbulEduBoxGetFile(QVariant&, QNetworkReply*)),
                              this, SLOT(rpcFault(int, const QString &)));
        QApplication::setOverrideCursor(Qt::WaitCursor);
        QApplication::processEvents();
    }
}

void AbulEduBoxFileManagerV1::on_btnCompletFermer_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Si on a un parent on se cache sinon on close
    if (m_parent)
    {
        hide();
    }
    else
    {
        close();
    }
    emit signalAbeFileCloseOrHide();
}

void AbulEduBoxFileManagerV1::on_btnSimpleFermer_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    on_btnCompletFermer_clicked();

}

void AbulEduBoxFileManagerV1::resizeEvent(QResizeEvent *)
{
    m_frmBandeauBackground->resize(width(),20);
}

void AbulEduBoxFileManagerV1::ifAuthenticateFailed()
{
    AbulEduMessageBoxV1* messageAide = new AbulEduMessageBoxV1(trUtf8("Restriction"),trUtf8("Cet espace n'est disponible qu'en étant authentifié..."));
    messageAide->show();
    on_btnCompletMonPC_clicked();
}

void AbulEduBoxFileManagerV1::on_tbRecherche_clicked()
{
    on_leRecherche_returnPressed();
}
