/** Classe AbulEduIdentitesNetworkV1 qui se charge des échanges réseaux
  *
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduidentitesnetworkv1.h"
#include "abuleduapplicationv1.h"

AbulEduIdentitesNetworkV1::AbulEduIdentitesNetworkV1(QObject *parent) :
    QObject(parent)
{
    m_localDebug = 0;
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_nam = abeApp->getAbeNetworkAccessManager();
    m_rpc = new MaiaXmlRpcClient(QUrl("https://webadminxml.abuledu.net/login.php"), this);
}

void AbulEduIdentitesNetworkV1::rpcFault(int code, const QString &errorMessage)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";

        qDebug() << "===================================";
        qDebug() << "AbulEduIdentitesNetworkV1::rpcFault : " << code << errorMessage;
        qDebug() << "===================================";
    }
}

void AbulEduIdentitesNetworkV1::rpcLoaded(QVariant &v, QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
        //        qDebug() << "===================================";
        //        qDebug() << "AbulEduIdentitesNetworkV1::rpcLoaded : " << v;
        //        qDebug() << "===================================";
    }

    emit signalServerReturnIdentite(v,reply);
}

void AbulEduIdentitesNetworkV1::rpcUpdated(QVariant &v, QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
        qDebug() << "===================================";
        qDebug() << "Requette de mise à jour : " << v;
        qDebug() << "===================================";
    }
    emit signalServerMessageAvailable(v.toMap().value("Message").toString());
}

void AbulEduIdentitesNetworkV1::slotGetMyLogsAsPDF()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }

#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
    QUrl url("https://leterrier.abuledu.net/export_logs.php");
    url.setUserName(m_nam->abeGetSSOLogin());
    url.setPassword(m_nam->abeGetSSOPassword());
#else
    QUrlQuery url("https://leterrier.abuledu.net/export_logs.php");
#endif

    //Si on est alacarte on recupere tous les logs, sinon c'est qu'on est lance depuis un logiciel
    //du genre mulot et on ne veut pas les logs d'imageo
    if(abeApp->applicationName() != "abuledu-alacarte") {
        url.addQueryItem("logiciel",abeApp->applicationName());
    }

    QNetworkReply *networkReplyDownload = m_nam->get(QNetworkRequest(url));
    networkReplyDownload->setObjectName("downloadLogs");

    m_downloadProgress = new AbulEduDownloadProgressBar(networkReplyDownload);
    m_downloadProgress->show();
    connect(networkReplyDownload,SIGNAL(finished()),this,SLOT(slotLogsPDFDownloaded()),Qt::UniqueConnection);
}

void AbulEduIdentitesNetworkV1::slotLogsPDFDownloaded()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(sender()->objectName() == "downloadLogs") {
        QNetworkReply *networkReplyDownload = (QNetworkReply*) sender();
        QString fic = m_downloadProgress->fileNameFromContentDisposition(networkReplyDownload->rawHeader("Content-Disposition"));
        if(fic == "") {
            fic = "tempNameAlaCarteLogs.pdf";
        }
        QString filename = QDir::tempPath() + "/" + fic;
//        qDebug() << "Download, save to " << filename;

        QFile file(filename);
        if (!file.open(QIODevice::WriteOnly)) {
            emit signalErrorPDFDownload(trUtf8("Erreur d'écriture du fichier : %1\n").arg(filename));
            networkReplyDownload->deleteLater();
            m_downloadProgress->close();
            return;
        }

        file.write(networkReplyDownload->readAll());
        file.close();

        networkReplyDownload->deleteLater();

        m_downloadProgress->close();
        //Maintenant on ouvre le fichier PDF
        emit signalPDFDownloaded(filename);
    }
}


void AbulEduIdentitesNetworkV1::abeUpdateIdentitie(QVariantMap tableau)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
        qDebug() << "  On y va avec courage et " << tableau;
    }
    //On fait une requête de mise à jour
    QList<QVariant> params = QList<QVariant>() << tableau;
    MaiaXmlRpcClient *rpc = new MaiaXmlRpcClient(QUrl("https://webadminxml.abuledu.net/login.php"),this);
    rpc->call("abuledu.sso.updateLogin", params,
             this, SLOT(rpcUpdated(QVariant&, QNetworkReply*)),
              this, SLOT(rpcFault(int, const QString &)));

}
