/** Classe AbulEduIdentitesUIV1
  *
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduidentitesuiv1.h"
#include "ui_abuleduidentitesuiv1.h"

AbulEduIdentitesUIV1::AbulEduIdentitesUIV1(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduIdentitesUIV1)
{
    m_localDebug = false;
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    ui->setupUi(this);
    m_abuleduIdentite = abeApp->getAbeIdentite();
    connect(m_abuleduIdentite,SIGNAL(signalLoaded()),this,SLOT(slotIdentitieLoaded()),Qt::UniqueConnection);
    connect(m_abuleduIdentite,SIGNAL(signalMessageAvailable(QString)),this,SLOT(slotMessageAvailable(QString)),Qt::UniqueConnection);
}

AbulEduIdentitesUIV1::~AbulEduIdentitesUIV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }

    delete ui;
}

void AbulEduIdentitesUIV1::abeLoad()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    ui->cbIdentite->setCurrentIndex(-1);
    ui->cbIdentite->clear();
    m_identites.clear();
}

void AbulEduIdentitesUIV1::abeRefresh()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";

        qDebug() << "==============================================================";
        qDebug() << "AbulEduIdentitesUIV1 On a " << m_abuleduIdentite->abeGetLogin() << " et " << abeApp->getAbeNetworkAccessManager()->abeGetSSOLogin();
        qDebug() << "==============================================================";
    }

    //Si c'est l'identifiant sous lequel je me suis authentifié je peux changer mes informations, sinon c'est en lecture seule
    if(m_abuleduIdentite->abeGetLogin() == abeApp->getAbeNetworkAccessManager()->abeGetSSOLogin()) {
        QList<QLineEdit *> allLE = findChildren<QLineEdit *>();
        QListIterator<QLineEdit*> i(allLE);
        while (i.hasNext()) {
            //qDebug() << "objet (a): " << i.next()->objectName();
            i.next()->setEnabled(true);
        }
    }
    else {
        QList<QLineEdit *> allLE = findChildren<QLineEdit *>();
        QListIterator<QLineEdit*> i(allLE);
        while (i.hasNext()) {
            //qDebug() << "objet (b): " << i.next()->objectName();
            i.next()->setEnabled(false);
        }
    }
    ui->leDomaine->setText(m_abuleduIdentite->abeGetDomaine());
    ui->leNom->setText(m_abuleduIdentite->abeGetNom());
    ui->lePrenom->setText(m_abuleduIdentite->abeGetPreNom());
    ui->leTelephone->setText(m_abuleduIdentite->abeGetTelephone());
    ui->leMobile->setText(m_abuleduIdentite->abeGetMobile());
    if(m_abuleduIdentite->abeGetEmail().size() > 0) {
        ui->leMail->setText(m_abuleduIdentite->abeGetEmail().at(0));
    }
    else {
        ui->leMail->setText("");
    }
    ui->leAdresse1->setText(m_abuleduIdentite->abeGetAdresse());
    ui->leAdresse2->setText(m_abuleduIdentite->abeGetCodePostal());
    ui->leAdresse3->setText(m_abuleduIdentite->abeGetVille());
    if(m_abuleduIdentite->abeGetResponsables().size() > 0)
        ui->leResponsable1->setText(m_abuleduIdentite->abeGetResponsables().at(0));
    ui->leLogin->setText(m_abuleduIdentite->abeGetLogin());
    if(!m_abuleduIdentite->abeGetPhoto()->isNull()) {
        if(m_localDebug) {
            qDebug() << "  on a une photo ...";
            qDebug() << "  et on la sauvegarde ici " << abeApp->getAbeHomepath() + "/leterrier/avatars/" + m_abuleduIdentite->abeGetLogin() + ".jpg";
        }
        QPixmap p(*m_abuleduIdentite->abeGetPhoto());
        QDir dir(abeApp->getAbeHomepath() + "/leterrier/avatars/");
        if(!dir.exists()) {
            dir.mkpath(abeApp->getAbeHomepath() + "/leterrier/avatars/");
        }
        p.save(abeApp->getAbeHomepath() + "/leterrier/avatars/" + m_abuleduIdentite->abeGetLogin() + ".jpg");
        ui->btnPhoto->setIconeNormale(abeApp->getAbeHomepath().replace("\\","/") + "/leterrier/avatars/" + m_abuleduIdentite->abeGetLogin() + ".jpg");
    }
    else {
        qDebug() << "On n'a pas de photo ...";
        ui->btnPhoto->setIconeNormale(":/abuleduidentitesv1/avatars/mystere");
    }

    if(m_abuleduIdentite->abeGetDomaine().contains("unifiedaccounts.zz")) {
        ui->cbIdentite->insertItem(0,trUtf8("%1 (identité principale)").arg(m_abuleduIdentite->abeGetLogin()));
    }
    else {
        ui->cbIdentite->insertItem(0,m_abuleduIdentite->abeGetLogin());
    }
}

AbulEduIdentitesV1 *AbulEduIdentitesUIV1::abeGetIdentite(QString login)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1) : " << login;
    }
    //On cherche dans la liste des identites s'il y en a une qui a email comme adresse email, si c'est le cas on la retourne
    for(int i = 0; i < m_identites.count(); i++) {
        if(m_localDebug) {
            qDebug() << "AbulEduIdentitesUIV1::abeGetIdentite : " << login << " pour " << i;
        }
        if(m_identites.at(i).data()->abeGetLogin() == login) {
            if(m_localDebug) {
                qDebug() << "AbulEduIdentitesUIV1::abeGetIdentite : " << i << " : " << m_identites.at(i).data()->abeGetLogin();
            }
            m_abuleduIdentite = m_identites.at(i);
        }
    }
    return m_abuleduIdentite;
}

void AbulEduIdentitesUIV1::on_btnUpdate_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }

    m_abuleduIdentite->abeSetMobile(ui->leMobile->text().trimmed());
    m_abuleduIdentite->abeSetTelephone(ui->leTelephone->text().trimmed());
    m_abuleduIdentite->abeSetNom(ui->leNom->text().trimmed());
    m_abuleduIdentite->abeSetPreNom(ui->lePrenom->text().trimmed());
    m_abuleduIdentite->abeSetEmail(ui->leMail->text().trimmed());
    m_abuleduIdentite->abeSetAdresse(ui->leAdresse1->text().trimmed());
    m_abuleduIdentite->abeSetCodePostal(ui->leAdresse2->text().trimmed());
    m_abuleduIdentite->abeSetVille(ui->leAdresse3->text().trimmed());

    m_abuleduIdentite->abeSetPhoto(new QPixmap(ui->btnPhoto->getIconeNormale()));

    QStringList resps;
    resps.append(ui->leResponsable1->text().trimmed());
    resps.append(ui->leResponsable2->text().trimmed());
    m_abuleduIdentite->abeSetResponsables(resps);

    //Si le mot de passe n'est pas vide *et* confirmé
    if(!ui->lePassword1->text().trimmed().isEmpty() &&
            (ui->lePassword1->text().trimmed() == ui->lePassword2->text().trimmed())
            ) {
        m_abuleduIdentite->abeSetPassword(ui->lePassword1->text().trimmed());
    }
    m_abuleduIdentite->abeUpdateIdentitie();
}


void AbulEduIdentitesUIV1::on_btnUnifier_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }


    //On affiche une popup qui demande le mail principal
    bool ok;
    QString text = QInputDialog::getText(this, trUtf8("Unification de compte"),
                                         trUtf8("Adresse email du compte principal:"), QLineEdit::Normal,
                                         QDir::home().dirName(), &ok);
    if (ok && !text.isEmpty()) {
        if(m_localDebug) {
            qDebug() << "Demande d'unification avec le compte mail " << text;
        }
        ui->lblMessage->document()->setPlainText(trUtf8("Demande d'unification du compte %1 avec %2 en cours.").arg(ui->lblLogin->text()).arg(text));

        //@todo 20121105
//        m_abuleduIdentite->abeUnifiedWith(text);
    }
}

void AbulEduIdentitesUIV1::on_cbIdentite_currentIndexChanged(int index)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1) : " << index;
    }

//    QString responsable;
//    if(m_identites.count() >= index && index >= 0) {
//        if(!m_identites.at(index).data()->abeGetResponsables().isEmpty())
//            responsable = m_identites.at(index).data()->abeGetResponsables().at(0);
//        this->refresh();
//    }
}

void AbulEduIdentitesUIV1::on_btnMesLogs_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    m_abuleduIdentite->abeGetMyLogsAsPDF();
}


void AbulEduIdentitesUIV1::slotChoixAvatar(QString avatar)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    ui->btnPhoto->setIconeNormale(avatar);
}

void AbulEduIdentitesUIV1::slotIdentitieLoaded()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    abeLoad();
    abeRefresh();
}

void AbulEduIdentitesUIV1::on_btnPhoto_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    AbulEduIdentitesAvatarsListingV1 *u = new AbulEduIdentitesAvatarsListingV1();
    u->move(mapToGlobal(ui->btnPhoto->pos()));
    u->show();
    connect(u,SIGNAL(signalChoixAvatar(QString)),this,SLOT(slotChoixAvatar(QString)), Qt::UniqueConnection);
}

void AbulEduIdentitesUIV1::slotMessageAvailable(QString message)
{
    //a remonter dans le ui
//    QString messageComplet(ui->lblMessage->document()->toPlainText());
//    messageComplet.append(message);
    ui->lblMessage->document()->setPlainText(message);
    QTimer::singleShot(3000,ui->lblMessage,SLOT(clear()));
}

void AbulEduIdentitesUIV1::on_lePassword1_textChanged(const QString &arg1)
{
    Q_UNUSED(arg1);

    if(ui->lePassword1->text() != ui->lePassword2->text()) {
        ui->btnUpdate->setEnabled(false);
    }
    else {
        ui->btnUpdate->setEnabled(true);
    }
}

void AbulEduIdentitesUIV1::on_lePassword2_textEdited(const QString &arg1)
{
    on_lePassword1_textChanged(arg1);
}
