/** Classe AbulEduIdentitesV1
  *
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduidentitesv1.h"

AbulEduIdentitesV1::AbulEduIdentitesV1(QObject *parent) :
    QObject(parent)
{
    m_localDebug = false;
    m_photo = new QPixmap();
    m_abuleduIdentiteNetwork = new AbulEduIdentitesNetworkV1(this);
    connect(m_abuleduIdentiteNetwork,SIGNAL(signalServerReturnIdentite(QVariant&,QNetworkReply*)),this,SLOT(slotSetFromNetworkReply(QVariant&,QNetworkReply*)),Qt::UniqueConnection);
    connect(m_abuleduIdentiteNetwork,SIGNAL(signalErrorPDFDownload(QString)),this,SLOT(slotErrorPDFDownload(QString)),Qt::UniqueConnection);
    connect(m_abuleduIdentiteNetwork,SIGNAL(signalPDFDownloaded(QString)),this,SLOT(slotPDFDownloaded(QString)),Qt::UniqueConnection);
    connect(m_abuleduIdentiteNetwork,SIGNAL(signalServerMessageAvailable(QString)), this, SLOT(slotMessageAvailable(QString)),Qt::UniqueConnection);
    connect(this,SIGNAL(signalGetMyLogsAsPDF()),m_abuleduIdentiteNetwork,SLOT(slotGetMyLogsAsPDF()),Qt::UniqueConnection);
}

AbulEduIdentitesV1::~AbulEduIdentitesV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }

}

void AbulEduIdentitesV1::abeUpdateIdentitie()
{
    QVariantMap tableau;
    tableau["mobile"]     = m_mobile;
    tableau["telephone"]  = m_telephone;
    tableau["nom"]        = m_nom;
    tableau["prenom"]     = m_prenom;
    tableau["email"]      = m_email.at(0);
    tableau["adresse"]    = m_adresse;
    tableau["codepostal"] = m_codepostal;
    tableau["ville"]      = m_ville;
    tableau["manager"]    = m_mes_responsables_legaux.at(0);
    tableau["motdepasse"] = m_password;

    QByteArray image;
    QBuffer buffer(&image);
    buffer.open(QIODevice::WriteOnly);
    m_photo->save(&buffer,"JPG");
    tableau["photo"]      = QString(image.toBase64());

    m_abuleduIdentiteNetwork->abeUpdateIdentitie(tableau);
}

void AbulEduIdentitesV1::abeSet(QString nom, QString prenom, QString domaine, QString telephone, QString mobile, QString email, QString adresse,
                                QString codepostal, QString ville, QStringList responsables, QString login, QPixmap *photo, QString groupes)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
        qDebug() << "On travaille sur l'objet suivant : " << objectName() << " pour la gestion de " << m_login << " nom : " << nom;
    }


    m_nom = nom;
    m_prenom = prenom;
    m_photo = photo;
    m_login = login;
    m_mobile = mobile;
    m_adresse = adresse;
    m_codepostal = codepostal;
    m_ville = ville;
    m_telephone = telephone;

    m_email.clear();
    m_email.append(email);

    m_domaine = domaine;
    m_mes_responsables_legaux = responsables;
    m_groupes = groupes.split(",");

    //    QString     commentaire;
    //    QString     usager_or_responsable_title;
    //    QString     usager_or_responsable_group;
    //    QString     unifiedEmail;
    //    QStringList groupes;
    //    QString     groupes_last_flag;
    //    QStringList responsabilites;
    //    QString     responsabilites_last_flag;
    //    QStringList invitations;
    //    QString     invitations_last_flag;
    //    QString     home;
    //    QString     uid;
    //    QString     gid;
    //    QString     profil;
    //    QString     config;
    //    QStringList system_logins;
    //    QString     quota_connex;
    //    QStringList responsable_legal_de_qui;
    //    QString     responsablelegalinvitationcode;
}



void AbulEduIdentitesV1::abeClear()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    m_nom.clear();
    m_prenom.clear();
    m_photo = new QPixmap();
    m_login.clear();
    m_mobile.clear();
    m_adresse.clear();
    m_codepostal.clear();
    m_ville.clear();
    m_telephone.clear();
    m_email.clear();
    m_domaine.clear();
    m_mes_responsables_legaux.clear();
    m_groupes.clear();
}

vCard AbulEduIdentitesV1::abeGetVCard()
{
    vCard vcard;
    vCardProperty name_prop  = vCardProperty::createName(m_prenom,m_nom);
    vcard.addProperty(name_prop);
    vCardProperty fn_prop    = vCardProperty(VC_FORMATTED_NAME, m_prenom+" "+m_nom);
    vcard.addProperty(fn_prop);
    vCardProperty note_prop  = vCardProperty(VC_NOTE, "");
    vcard.addProperty(note_prop);
    vCardProperty org_prop   = vCardProperty::createOrganization("");
    vcard.addProperty(org_prop);
    vCardProperty mail_prop   = vCardProperty(VC_EMAIL, m_email);
    vcard.addProperty(mail_prop);
    /** Structure du champ adresse : BP;adresse;rue;ville;région;codepostal;pays */
    vCardProperty adress_prop   = vCardProperty(VC_ADDRESS, ";"+m_adresse+";"+";"+m_ville+";"+";"+m_codepostal+";");
    vcard.addProperty(adress_prop);
    vCardProperty telephone_prop   = vCardProperty(VC_TELEPHONE, m_telephone);
    vcard.addProperty(telephone_prop);
    vCardProperty url_prop   = vCardProperty(VC_URL, "");
    vcard.addProperty(url_prop);

    return vcard;
}

void AbulEduIdentitesV1::slotSetFromNetworkReply(QVariant &v, QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
        //        qDebug() << "===================================";
        //        qDebug() << "AbulEduIdentitesNetworkV1::rpcLoaded : " << v;
        //        qDebug() << "===================================";
    }

    for(int i = 0; i < v.toMap().count(); i++) {
        QMap<QString,QVariant> monItem = v.toMap().value(QString::number(i)).toMap();
        if(monItem.count() > 0 && (monItem.value("login").toString() != "")) {
            if(m_localDebug) {
                qDebug() << "***********************************";
                qDebug() << "ITEM : " << monItem;
                qDebug() << "***********************************";
            }
            QPixmap *photo = new QPixmap();
            if(!photo->loadFromData(QByteArray::fromBase64(monItem.value("photo").toByteArray()))) {
                if(m_localDebug) {
                    qDebug() << "Erreur de chargement de la photo ...";
                }
            }
            //On enquille toutes les identités les unes apres les autres dans notre structure de données
            abeSet(monItem.value("nom").toString(), monItem.value("prenom").toString(), monItem.value("domaine").toString(),
                   monItem.value("telephone").toString(), monItem.value("mobile").toString(), monItem.value("email").toString(),
                   monItem.value("adresse").toString(), monItem.value("codepostal").toString(), monItem.value("ville").toString(),
                   monItem.value("manager").toStringList(), monItem.value("login").toString(), photo, monItem.value("groupes").toStringList().join(","));
        }
    }
    emit signalLoaded();
}

void AbulEduIdentitesV1::abeGetMyLogsAsPDF()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    if(m_login != "") {
        emit signalGetMyLogsAsPDF();
    }
    else {
        QMessageBox::warning(0, trUtf8("Erreur de téléchargement"),
                             trUtf8("Vous devez être authentifié pour télécharger votre journal d'activités AbulÉdu"),
                             QMessageBox::Ok);
    }
}

void AbulEduIdentitesV1::slotErrorPDFDownload(QString message)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    QMessageBox::warning(0, trUtf8("Erreur de téléchargement"),
                         message,
                         QMessageBox::Ok);
    emit signalMessageAvailable(message);
}

void AbulEduIdentitesV1::slotPDFDownloaded(QString filename)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
    }
    QDesktopServices::openUrl(QUrl(filename, QUrl::TolerantMode));
}

void AbulEduIdentitesV1::slotMessageAvailable(QString message)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
        qDebug() << "  " << message;
    }
    emit signalMessageAvailable(message);
}


//void AbulEduIdentitesV1::abeUnifiedWith(QString email)
//{
//    if(m_localDebug) {
//        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << " (1)";
//    }
//    //On fait une demande d'unification du compte
//    //La structure de mise à jour
//    QVariantMap tableau;
//    tableau["unifiedMail"] = email;
//    QList<QVariant> params = QList<QVariant>() << tableau;

//    m_rpc->call("abuledu.sso.updateLoginUnifiedEmail", params,
//                this, SLOT(rpcUpdated(QVariant&, QNetworkReply*)),
//                this, SLOT(rpcFault(int, const QString &)));

//}

