/** Classe AbulEduMessageBoxV1
  *
  * @author 2012 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledumessageboxv1.h"
#include "ui_abuledumessageboxv1.h"

AbulEduMessageBoxV1::AbulEduMessageBoxV1(QString titre, QString texte, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduMessageBoxV1)
{
    m_localDebug = 0;
    m_parent = parent;
    premierAffichage = false;
    ui->setupUi(this);

    m_config = new QSettings(":/abuledumessageboxv1/settings/conf", QSettings::IniFormat);
    setTitleColor(QColor(m_config->value("colors/title").toString()));

    ui->lblTitre->setFont(QFont(abeApp->font().family(),ui->lblTitre->font().pointSize()));
    ui->lblTitre->setText(titre);
    ui->lblTexte->setFont(QFont(abeApp->font().family(),ui->lblTexte->font().pointSize()));
    ui->lblTexte->setText(texte);
    setAttribute( Qt::WA_TranslucentBackground, true);
    setWindowFlags(Qt::FramelessWindowHint | Qt::SubWindow);

    //Surtout pas sinon les logiciels "plantent": ils font un new dans le constructeur mais ensuite juste des show ... et nous dans le close
    //on se suicide ... donc le 2° show -> plantage
    //    setAttribute(Qt::WA_DeleteOnClose);

    ui->btnFermer->setFont(abeApp->font());
    ui->btnFermer->setCouleurTexteNormale(QColor(233,45,0));
    ui->btnFermer->setCouleurTexteSurvol(QColor(249,80,0));
//    ui->btnFermer->setFont(QFont(ui->btnFermer->font().family(),1.4*ui->btnFermer->font().pointSize(),QFont::DemiBold));
    ui->btnFermer->setIconeNormale(QString(":/abuledumessageboxv1/button/close"));
//    ui->btnFermer->setFixedHeight(QPixmap(":/abuledumessageboxv1/button/close").height());
//    ui->btnFermer->setTextePaddingLeft(3*ui->btnFermer->font().pointSize());
    if(m_parent == 0) {
        // On en fait une fenetre modale par défaut
        setWindowModality(Qt::ApplicationModal);
        raise();
    }

    //On centre l'AbulEduMessageBoxV1
    if(m_parent != 0)
    {
        move(m_parent->geometry().center().x() - width()/2, m_parent->geometry().center().y() - height()/2);
    }
    else
    {
        QDesktopWidget *widget = QApplication::desktop();
        int desktop_width = widget->width();
        int desktop_height = widget->height();
        this->move((desktop_width-this->width())/2, (desktop_height-this->height())/2);
    }
}

AbulEduMessageBoxV1::~AbulEduMessageBoxV1()
{
    delete ui;
}

void AbulEduMessageBoxV1::setTitleColor(QColor color)
{
    ui->lblTitre->setStyleSheet(ui->lblTitre->styleSheet().replace(ui->lblTitre->styleSheet().indexOf("#"),7,color.name()));
}

void AbulEduMessageBoxV1::setWink()
{
    ui->lblIcone->setPixmap(QPixmap(":/abuledumessageboxv1/icon/wink"));
}

void AbulEduMessageBoxV1::hideCloseButton()
{
    ui->btnFermer->hide();
    resize(width(),height()-ui->btnFermer->height());
}

void AbulEduMessageBoxV1::on_btnFermer_clicked()
{
    emit signalFermeture();
    close();
}
