/**
  * Classe 
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledunetworktests.h"

AbulEduNetworkTests::AbulEduNetworkTests(QObject *parent) :
    QObject(parent)
{
    m_nbPingTotal = 10;
    m_nbPingCurrent = 0;
    m_timerTests = new QTimer(this);
    m_timerTimeOut = new QTimer(this);
}

void AbulEduNetworkTests::abeRunTest(QString domainTest, int portTest, int nbTests)
{
//    qDebug() << "abeRunTest";

    m_nbPingTotal   = nbTests;
    m_portTest      = portTest;
    m_domainTest    = domainTest;
    m_nbPingCurrent = 0;

    m_socket.setSocketOption(QAbstractSocket::LowDelayOption,1);
    m_socket.connectToHost(m_domainTest,m_portTest);

    connect(&m_socket,SIGNAL(error(QAbstractSocket::SocketError)),this,SLOT(slotError(QAbstractSocket::SocketError)));
    connect(&m_socket,SIGNAL(connected()),this,SLOT(slotConnected()));

    //Toutes les 5 secondes on relance le test de connexion
    m_timerTests->setInterval(5000);
    connect(m_timerTests,SIGNAL(timeout()),this,SLOT(slotRestartTest()));
    m_timerTests->start();

    //Et si au bout de 4 secondes pas de réponse on stoppe
    m_timerTimeOut->setInterval(4000);
    connect(m_timerTimeOut,SIGNAL(timeout()),this,SLOT(slotError()));
    m_timerTimeOut->start();
    m_time.start();
}

void AbulEduNetworkTests::abeStopTest()
{
    m_timerTimeOut->stop();
    m_timerTests->stop();
}

void AbulEduNetworkTests::slotRestartTest()
{
//    qDebug() << "slotRestartTest";

    m_timerTests->stop();
    m_timerTimeOut->stop();

    m_socket.close();
    m_socket.connectToHost(m_domainTest,m_portTest);

    m_timerTests->start();
    m_timerTimeOut->start();
    m_time.start();
}

void AbulEduNetworkTests::slotConnected()
{
    m_timerTimeOut->stop();
    m_socket.close();
    emit signalOk(m_time.elapsed());
//    qDebug() << "Connected : " <<  m_time.elapsed() << "ms";
}

void AbulEduNetworkTests::slotError()
{
    m_timerTimeOut->stop();
    m_socket.abort();
    m_socket.close();

    emit signalError(1,m_time.elapsed());
//    qDebug() << "Error : " <<  m_time.elapsed() << "ms";
}

void AbulEduNetworkTests::slotError(QAbstractSocket::SocketError socketError)
{
    m_timerTimeOut->stop();
    m_socket.abort();
    m_socket.close();

    emit signalError(2,m_time.elapsed());
//    qDebug() << "Error : " <<  m_time.elapsed() << "ms";
}

