/** Classe AbulEduSSOLoginV1 : gestion de l'authentification SSO AbulEdu
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledussologinv1.h"
#include "ui_abuledussologinv1.h"

AbulEduSSOLoginV1::AbulEduSSOLoginV1(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduSSOLoginV1)
{
    ui->setupUi(this);
    //A passer a true si vous voulez les infos de debug de ce module
    m_localDebug = false;
    m_moving = false;

    m_abuleduNetworkTest = new AbulEduNetworkTests(this);
    connect(m_abuleduNetworkTest,SIGNAL(signalError(int,int)), this, SLOT(slotNetworkTestError(int,int)));
    connect(m_abuleduNetworkTest,SIGNAL(signalOk(int)), this, SLOT(slotNetworkTestOk(int)));

    m_petitOuGrand = abePetitSSO;
    ui->stackedWidgetPrincipal->setCurrentWidget(ui->pagePetitSSO);
    ui->btnTetiereSimple->setIconeNormale(":/abuledussologinv1/tetiere-simple-anonyme");

    // En attentant d'avoir des machines a état partout, on fait comme papy et on essaye de gérer les quelques
    // malheureux composants et leur comportement à la main ... moins de 10, on devrait y arriver !
    ui->btnLogout->hide();
    ui->frmLoginMessage->hide();
    ui->AbulEduSSOLoginStackedWidget->setCurrentWidget(ui->pageLoginLogout);


    // On récupère le network access manager
    m_nam = abeApp->getAbeNetworkAccessManager();
    connect(m_nam, SIGNAL(ssoAuthStatus(int)), this, SLOT(slotSSOAuthStatus(int)), Qt::UniqueConnection);

    // Un flag pour savoir si l'action de login ou logout est en cours
    m_logoutInProgress = false;

    // Les URL magiques pour changer le mot de passe
    m_rpcWebadminResetPass = new MaiaXmlRpcClient(QUrl("https://webadminxml.abuledu.net/resetpassword.php"), this);

    // Es-ce qu'on propose de s'authentifier sous le nom qu'on a entré la dernière fois ?
    // après avoir demandé à l'équipe la réponse est "oui", on propose même les 5 dernières identités !
    QSettings ssoLoginSettings("abuledu","ssologin");
    QStringList loginDejaUtilises(ssoLoginSettings.value("LastLogins").toStringList());
    QCompleter *loginCompleter = new QCompleter(loginDejaUtilises, this);
    loginCompleter->setCaseSensitivity(Qt::CaseInsensitive);
    ui->leLogin->setCompleter(loginCompleter);

    ui->cbLoginDomaine->insertItem(0,trUtf8("Adresse email principale"),"mail_principal");
    QStringList domainesDejaUtilises(ssoLoginSettings.value("LastDomains").toStringList());
    for(int i = 0; i < domainesDejaUtilises.size(); i++) {
        ui->cbLoginDomaine->insertItem(i+1,domainesDejaUtilises.at(i));
    }


    // Si on n'a pas de parent c'est qu'on est une popup, de ce fait on se comporte comme un splashscreen
    // le maitre des arts graphiques nous ayant demandés de faire un truc dont il aurait pas trop honte ...
    if(parent == 0) {
        if(m_localDebug) {
            qDebug()  << "  " __FILE__ <<  __LINE__ << __FUNCTION__ << "  sans parent: on ouvre une popup ...";
        }
        //pas de |Qt::SplashScreen); sinon un clic dessus le cache !
        //et pas de |Qt::WindowStaysOnTopHint|Qt::X11BypassWindowManagerHint); sinon on garde la sso sur tous les bureaux linux !
        setWindowFlags(Qt::FramelessWindowHint);
        setWindowState(Qt::WindowActive);

        //On centre la fenetre
//        QRect frect = frameGeometry();
//        frect.moveCenter(QDesktopWidget().availableGeometry().center());
//        ui->frameLogin->move(frect.topLeft());

        //hack rapide 2012 12 11 pour tester la popup sso qui bloque tout l'écran et le passe en gris
        QImage *img = new QImage(QPixmap::grabWindow(QApplication::desktop()->winId()).toImage());
        QPixmap background = QPixmap::fromImage(*greyScale(img));
        QPalette palette;
        QBrush brush(background);
        palette.setBrush(QPalette::Background,brush);
        setPalette(palette);

//        background.save("/tmp/toto.png");
//        QPainter painter(ui->framePrincipale);
//        painter.drawPixmap(0, 0, background, 0, 0, 100, 100);


        abeApp->setActiveWindow(this);
        activateWindow();
        raise();
        abeFocus();
//        m_pixmap = QPixmap(":/abuledussologinv1/cadre-identification");
//        setAttribute(Qt::WA_TranslucentBackground);
//        setFixedSize(m_pixmap.size());

        //Evite d'avoir le fond noir quand on est sous linux sans compiz
//        this->setMask(m_pixmap.mask());
//        ui->frameLogin->setStyleSheet("QObject { \nbackground: transparent; \n}");

        //Si on s'ouvre comme une fenetre -> centré et modal
//        move(QApplication::desktop()->screen()->rect().center() - rect().center());
//        setWindowModality(Qt::ApplicationModal);
    }
    else {
        if(m_localDebug) {
            qDebug()  << "  " __FILE__ <<  __LINE__ << __FUNCTION__ << "  avec parent: on s'integre dans le paysage ...";
        }
        //Dans le cas contraire c'est qu'on est embarqué en tant que widget, il faut donc s'assurer que le bouton
        //close n'est pas visible
        ui->btnClose->hide();
    }


    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
}

AbulEduSSOLoginV1::~AbulEduSSOLoginV1()
{
    delete ui;
    m_abuleduNetworkTest->deleteLater();
}

void AbulEduSSOLoginV1::abeFocus()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_petitOuGrand == abePetitSSO) {
        ui->leLogin->setFocus();
        setFocusPolicy(Qt::StrongFocus);
    }
}

void AbulEduSSOLoginV1::abeAddDomaineForNextLogin(QString domaine)
{
    domaine.remove(".auth.abuledu.net");
    //Et on sauvegarde le domaine dans la liste
    QSettings ssoLoginSettings("abuledu","ssologin");
    QStringList domainesDejaUtilises(ssoLoginSettings.value("LastDomains").toStringList());
    //Si on en a déjà 5 on supprime le 1er (le plus ancien) et on enquille les autres
    if(domainesDejaUtilises.count() > 5)
        domainesDejaUtilises.takeFirst();
    if(!domainesDejaUtilises.contains(domaine))
        domainesDejaUtilises << domaine;
    ssoLoginSettings.setValue("LastDomains", domainesDejaUtilises);
    ssoLoginSettings.sync();
}

QStringList AbulEduSSOLoginV1::abeGetDomainesList()
{
    QSettings ssoLoginSettings("abuledu","ssologin");
    return ssoLoginSettings.value("LastDomains").toStringList();
}

bool AbulEduSSOLoginV1::abeDelDomaine(QString domaine)
{
    //Au cas où
    domaine.remove(".auth.abuledu.net");
    //Et on sauvegarde le domaine dans la liste
    QSettings ssoLoginSettings("abuledu","ssologin");
    QStringList domainesDejaUtilises(ssoLoginSettings.value("LastDomains").toStringList());
    domainesDejaUtilises.removeOne(domaine);
    ssoLoginSettings.setValue("LastDomains", domainesDejaUtilises);
    ssoLoginSettings.sync();
}

void AbulEduSSOLoginV1::abeSetPetitOuGrandEnum(AbulEduSSOLoginV1::enumAbulEduSSOLOgin petitOuGrand)
{
    m_petitOuGrand = petitOuGrand;
    if(petitOuGrand == abePetitSSO) {
        ui->stackedWidgetPrincipal->setCurrentWidget(ui->pagePetitSSO);
    }
    else {
        ui->stackedWidgetPrincipal->setCurrentWidget(ui->pageGrandSSO);
        actualiseListeVignettesUtilisateurs();
    }
}

void AbulEduSSOLoginV1::rpcNewPasswordRequested(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug() << "============================================================";
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << trUtf8("Détails de la réponse newPassword:");
        qDebug() << v.toMap();
        qDebug() << "============================================================";
    }
    ui->lblLoginMessage->setText(v.toMap().value("Message").toString());
    //Si le message de retour commance par 2 c'est que tout va bien
    if(v.toMap().value("status").toString().startsWith("20")) {
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));
        m_situation = "password_par_email_envoye";
        ui->btnLogin->setText(trUtf8("Je me connecte"));
        ui->lePassword->show();
        ui->lblPassword->show();
    }
    else {
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/erreur"));
        m_situation = "password_par_email_erreur";
    }
    ui->frmLoginMessage->show();
}

void AbulEduSSOLoginV1::rpcNewAccountRequested(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug() << "============================================================";
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << trUtf8("Détails de la réponse newPassword:");
        qDebug() << v.toMap();
        qDebug() << "============================================================";
    }
    ui->lblLoginMessage->setText(v.toMap().value("Message").toString());
    //Si le message de retour commance par 2 c'est que tout va bien
    if(v.toMap().value("status").toString().startsWith("20")) {
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));
    }
    else {
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/erreur"));
    }
    ui->frmLoginMessage->show();
    ui->leLogin->setText(ui->leMailOuvrirCompte->text());
    ui->lePassword->setText("");
}

void AbulEduSSOLoginV1::rpcFault(int code, const QString &errorMessage)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //Tentative de réouverture d'une session SSO en cas de timeout
    if(code == -32300) {
        m_nam->abeSSOLogin();
    }

}

void AbulEduSSOLoginV1::slotSSOAuthStatus(int code)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QApplication::restoreOverrideCursor();

    if(code == 1) {
        //Tout est OK
        ui->btnLogin->setText("Changer d'utilisateur");
        ui->btnLogin->setIcon(QIcon(":/abuledussologinv1/changer"));
        ui->btnLogin->setEnabled(false);
        ui->btnLogout->show();
        ui->lePassword->setText("");
        ui->lblLoginMessage->setText(trUtf8("Connexion réussie pour l'utilisateur %1.").arg(ui->leLogin->text()));
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));
        ui->frmLoginMessage->show();

        //Et on sauvegarde le login dans la liste
        QSettings ssoLoginSettings("abuledu","ssologin");
        QStringList loginDejaUtilises(ssoLoginSettings.value("LastLogins").toStringList());
        //Si on en a déjà 30 on supprime le 1er (le plus ancien) et on enquille les autres
        if(loginDejaUtilises.count() > 30)
            loginDejaUtilises.takeFirst();
        if(!loginDejaUtilises.contains(ui->leLogin->text()))
            loginDejaUtilises << ui->leLogin->text();
        ssoLoginSettings.setValue("LastLogins", loginDejaUtilises);

        //Fermeture automatique de la fenêtre
        if(parent() == 0) {
            hide();
        }
    }
    else {
        //Si on est en phase de logout ...
        if(m_logoutInProgress) {
            //Ou si erreur
            ui->lblLoginMessage->setText(trUtf8("Session terminée. Vous pouvez maintenant vous authentifier sous une autre identité."));
            ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));
            m_logoutInProgress = false;
        }
        else {
            //Ou si erreur
            ui->lblLoginMessage->setText(trUtf8("Identifiant ou mot de passe incorrect ..."));
            ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/erreur"));

        }
        ui->frmLoginMessage->show();
        ui->btnLogin->setEnabled(true);
        ui->btnLogout->hide();
        //On resette aussi l'identité !
        abeApp->getAbeIdentite()->abeClear();
    }
    m_abuleduNetworkTest->abeStopTest();
    emit signalSSOAuthDone(code);
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::slotSSOAuthStatus (2)";
    }
}

void AbulEduSSOLoginV1::slotNetworkTestError(int code, int elapsedTime)
{
//    qDebug() << "slotNetworkTestError(int code, int elapsedTime)" << code << " et " << elapsedTime;
    ui->lblServeurPing->setPixmap(QPixmap(":/abuledussologinv1/serveur-rouge"));
    ui->lblServeurPing->setToolTip(trUtf8("Impossible de se connecter au serveur d'authentification"));
    //ui->btnLogin->setEnabled(false);
}

void AbulEduSSOLoginV1::slotNetworkTestOk(int elapsedTime)
{
//    qDebug() << "slotNetworkTestOk(int elapsedTime) " << elapsedTime;
    ui->lblServeurPing->setPixmap(QPixmap(":/abuledussologinv1/serveur-vert"));
    ui->lblServeurPing->setToolTip(trUtf8("Serveur d'authentification accessible en %1 ms").arg(elapsedTime));
    //ui->btnLogin->setEnabled(true);
}

void AbulEduSSOLoginV1::on_btnLogin_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //Si on est dans le cas de figure ou on demande a recuperer son mot de passe
    if(m_situation == "demande_password_par_email") {
        if(ui->leLogin->text() != "") {
            ui->frmLoginMessage->hide();
            ui->AbulEduSSOLoginStackedWidget->setCurrentWidget(ui->pageLoginLogout);

            qDebug() << "demande_password_par_email";
            //Mot de passe oublié, merci de m'en envoyer un nouveau ... requete a faire vers le webadmin/portail
            //On peut essayer de se lancer direct sur le XMLRPC
            QList<QVariant> params;
            params << ui->leLogin->text().trimmed();
            m_rpcWebadminResetPass->call("abuledu.resetpassword.sendNewPassword", params,
                                         this, SLOT(rpcNewPasswordRequested(QVariant&, QNetworkReply*)),
                                         this, SLOT(rpcFault(int, const QString &)));
        }
        else {
            ui->lblLoginMessage->setText(trUtf8("Veuillez entrer une adresse email ..."));
            ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));
        }
    }
    //Sinon on est dans le cas normal ou on demande à ouvrir son compte
    else if(ui->leLogin->text().trimmed() != "" && ui->lePassword->text().trimmed() != "") {
        ui->frmLoginMessage->hide();
        ui->AbulEduSSOLoginStackedWidget->setCurrentWidget(ui->pageLoginLogout);

        //Si on est déjà authentifié avec ce compte
        if(m_nam->abeSSOAuthenticationStatus() == 1) {
            if(m_nam->abeGetSSOLogin() == ui->leLogin->text() &&
                    m_nam->abeGetSSOPassword() == ui->lePassword->text()) {
                ui->lblLoginMessage->setText(trUtf8("Vous êtes déjà authentifié(e) avec ce compte utilisateur. Si vous voulez vous authentifier avec un autre compte, changez les valeurs dans les champs identifiants et mot de passe."));
                ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/attention"));

                ui->frmLoginMessage->show();
            }
        }
        //Ou alors, changement de compte ou authentification initiale
        else {
            QApplication::setOverrideCursor(Qt::WaitCursor);
            //On passe en prod sur la boutique en ligne ...
            m_webadminPassAsked = 0;
            //1. Authentification
            QString lelogin = ui->leLogin->text();
            //Si on a un domaine on concatène
            if(ui->cbLoginDomaine->itemData(ui->cbLoginDomaine->currentIndex(),Qt::UserRole) != "mail_principal") {
                lelogin += "@" + ui->cbLoginDomaine->currentText();
            }
//            qDebug() << " ++++++++++++++++++++++++++ " << lelogin;
            m_nam->setAbeLoginPass(lelogin, ui->lePassword->text());
            //On deconnecte une eventuelle connexion precedente
            m_nam->abeSSOLogin();
        }
    }
    else {
        ui->frmLoginMessage->show();
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/attention"));
        ui->lblLoginMessage->setText(trUtf8("Veuillez entrer un identifiant et un mot de passe ..."));
        ui->leLogin->setFocus();
    }
}

void AbulEduSSOLoginV1::on_btnPasswordOublie_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    ui->AbulEduSSOLoginStackedWidget->setCurrentWidget(ui->pageLoginLogout);

    ui->frmLoginPass->show();
    ui->frmLoginMessage->show();
    ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/attention"));
    ui->lblLoginMessage->setText(trUtf8("Veuillez entrer votre adresse email ..."));
    ui->lblPassword->hide();
    ui->lePassword->hide();
    ui->leLogin->setText("");
    ui->btnLogin->setText(trUtf8("Envoyer"));
    ui->btnLogin->show();
    ui->leLogin->setFocus();
    m_situation = "demande_password_par_email";
}

void AbulEduSSOLoginV1::on_btnOuvrirUnCompte_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Formulaire d'ouverture de compte ... sur la boutique ?
    //on affiche un mini formulaire de demande d'adresse email
    ui->frmLoginPass->hide();
    ui->btnLogin->hide();
    ui->btnLogout->hide();
    ui->frmLoginMessage->hide();
    ui->AbulEduSSOLoginStackedWidget->setCurrentWidget(ui->pageCreerCompte);
}

void AbulEduSSOLoginV1::on_btnLogout_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_logoutInProgress = true;
    ui->AbulEduSSOLoginStackedWidget->setCurrentWidget(ui->pageLoginLogout);

    ui->frmLoginMessage->hide();
    m_nam->abeSSOLogout();
}

void AbulEduSSOLoginV1::on_btnCreerUnCompte_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->leMailOuvrirCompte->text().trimmed() == "") {
        ui->frmLoginMessage->show();
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/attention"));
        ui->lblLoginMessage->setText(trUtf8("Veuillez entrer une adresse email valide ..."));
        ui->leMailOuvrirCompte->setFocus();
    }
    else {
        ui->AbulEduSSOLoginStackedWidget->hide();
        ui->frmBasPopup->hide();
        QList<QVariant> params;
        params << ui->leMailOuvrirCompte->text().trimmed();
        m_rpcWebadminResetPass->call("abuledu.resetpassword.createNewAccount", params,
                                     this, SLOT(rpcNewAccountRequested(QVariant&, QNetworkReply*)),
                                     this, SLOT(rpcFault(int, const QString &)));
    }
    //1. côté serveur -> vérifie que l'email est ok
    //1.bis le serveur retourne un mot de passe dans son XMLRPC (comme ça l'utilisateur peut bosser tout de suite)
    //1.ter envoie un mail avec une adresse pour confirmer l'ouverture du compte
    //2. lorsque l'utilisateur confirme l'ouverture du compte -> ouvre le compte sur la boutique
    //2.bis et demande prénom + nom
    //2.ter affiche offre spéciale
}

void AbulEduSSOLoginV1::showEvent(QShowEvent *event)
{
    Q_UNUSED(event);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if(m_nam->abeSSOAuthenticationStatus() == 1) {
        ui->btnLogin->setText("Changer d'utilisateur");
        ui->btnLogin->setIcon(QIcon(":/abuledussologinv1/changer"));
        ui->btnLogin->setEnabled(false);
        ui->btnLogout->show();
    }
    else {
        ui->btnLogin->setEnabled(true);
        ui->btnLogout->hide();
    }
    m_abuleduNetworkTest->abeRunTest("auth.abuledu.net",443,10);
}

void AbulEduSSOLoginV1::on_btnClose_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_abuleduNetworkTest->abeStopTest();

    emit signalClosed();
    close();
}

void AbulEduSSOLoginV1::on_lePassword_returnPressed()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    on_btnLogin_clicked();
}

void AbulEduSSOLoginV1::mousePressEvent(QMouseEvent* event)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if((event->button() == Qt::LeftButton) && ui->lblTitre->underMouse()) {
        m_moving = true;
        m_offset = event->pos();
    }
}

void AbulEduSSOLoginV1::mouseMoveEvent(QMouseEvent* event)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if(m_moving) {
        move(event->globalPos() - m_offset);
    }
}

void AbulEduSSOLoginV1::mouseReleaseEvent(QMouseEvent* event)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if(event->button() == Qt::LeftButton) {
        m_moving = false;
    }
}

QImage * AbulEduSSOLoginV1::greyScale(QImage * origin){
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QImage * newImage = new QImage(origin->width(), origin->height(), QImage::Format_ARGB32);
    QRgb col;
    int gray;
    for(int y = 0; y<newImage->height(); y++){
        for(int x = 0; x<newImage->width(); x++){
            col = origin->pixel(x,y);
            gray = qGray(col);
            newImage->setPixel(x,y, qRgba(gray, gray, gray, 50));
        }

    }

    return newImage;
}


void AbulEduSSOLoginV1::on_btnTetiereSimple_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //On passe sur l'affichage des vignettes des utilisateurs
    ui->stackedWidgetPrincipal->setCurrentWidget(ui->pageGrandSSO);

    actualiseListeVignettesUtilisateurs();
}

void AbulEduSSOLoginV1::actualiseListeVignettesUtilisateurs()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Uniquement si on a pas déjà affiché tous les comptes ...
    if(ui->lwGrandSSO->count() > 0) {
        return;
    }

    QSettings ssoLoginSettings("abuledu","ssologin");
    QStringList loginDejaUtilises(ssoLoginSettings.value("LastLogins").toStringList());
    for(int i = 0; i < loginDejaUtilises.count(); i++) {
        QListWidgetItem *itemIcone = new QListWidgetItem(loginDejaUtilises.at(i).split("@").at(0));
        itemIcone->setData(Qt::UserRole+1,loginDejaUtilises.at(i));

        //On cache le texte ... c'est rapide et le moins pire que j'ai trouvé
        //QFont invisible(abeApp->font().family(),12);
        //itemIcone->setFont(invisible);
        QFileInfo ficone;
        ficone.setFile(abeApp->getAbeHomepath() + "/leterrier/avatars/" + loginDejaUtilises.at(i) + ".jpg");
        //Si l'icone n'existe pas on ...
        QIcon icone;
        if(ficone.exists()) {
            icone = QIcon(ficone.absoluteFilePath());//pour la mettre  à coté de l'item
        }
        else {
            icone = QIcon(":/abuleduidentitesv1/avatars/mystere");//pour la mettre  à coté de l'item
        }
        itemIcone->setIcon(icone); // ajout de la petite icone sur l'item
        ui->lwGrandSSO->insertItem(i,itemIcone);
    }
}

void AbulEduSSOLoginV1::affichageParDefaut()
{
    ui->AbulEduSSOLoginStackedWidget->setCurrentWidget(ui->pageLoginLogout);
    ui->frmLoginPass->show();
    ui->btnLogin->show();
    ui->lblLogin->show();
    ui->lblPassword->show();
    ui->leLogin->show();
    ui->lePassword->show();
    ui->lblLoginMessage->hide();
    ui->lblLoginPicto->hide();
}

void AbulEduSSOLoginV1::on_lwGrandSSO_itemDoubleClicked(QListWidgetItem *item)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    affichageParDefaut();
    ui->stackedWidgetPrincipal->setCurrentWidget(ui->pagePetitSSO);
    ui->leLogin->setText(item->data(Qt::UserRole+1).toString());
    ui->lePassword->setFocus();
}

void AbulEduSSOLoginV1::on_btnTetiereSimpleRetourPetitSSO_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //On passe sur l'affichage normal
    affichageParDefaut();
    ui->stackedWidgetPrincipal->setCurrentWidget(ui->pagePetitSSO);
    ui->AbulEduSSOLoginStackedWidget->setCurrentWidget(ui->pageLoginLogout);
}
