/** Classe AbulEduSSOLoginV1 : gestion de l'authentification SSO AbulEdu
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUSSOLOGINV1_H
#define ABULEDUSSOLOGINV1_H

#include <QPainter>
#include <QBitmap>
#include <QWidget>
#include <QDesktopWidget>
#include <QCompleter>
#include <QMouseEvent>
#include <QListWidgetItem>
#include "abuledunetworkaccessmanagerv1.h"
#include "maiaXmlRpcClient.h"
#include "abuleduapplicationv1.h"
#include "abuledunetworktests.h"

namespace Ui {
    class AbulEduSSOLoginV1;
}

class AbulEduSSOLoginV1 : public QWidget
{
    Q_OBJECT
    Q_ENUMS(enumAbulEduSSOLOgin)

public:
    enum enumAbulEduSSOLOgin { abePetitSSO, abeGrandSSO };

    /**
      * Constructeur
      */
    explicit AbulEduSSOLoginV1(QWidget *parent = 0);
    ~AbulEduSSOLoginV1();
    void abeFocus();
    void abeAddDomaineForNextLogin(QString domaine);
    QStringList abeGetDomainesList();
    bool abeDelDomaine(QString domaine);

    /**
      * Permet de basculer d'une sso login simple a une sso login avec tt les vignettes
      * @param petitOuGrand cf enumAbulEduSSOLOgin
      */
    void abeSetPetitOuGrandEnum(enumAbulEduSSOLOgin petitOuGrand);

signals:
    void signalSSOAuthDone(int code);
    void signalClosed();

private slots:
    void actualiseListeVignettesUtilisateurs();
    void affichageParDefaut();
    void showEvent ( QShowEvent * event );
    void rpcNewPasswordRequested(QVariant &v, QNetworkReply *reply);
    void rpcNewAccountRequested(QVariant &v, QNetworkReply *reply);
    void rpcFault(int code, const QString &errorMessage);
    void slotSSOAuthStatus(int code);
    void slotNetworkTestError(int code, int elapsedTime);
    void slotNetworkTestOk(int elapsedTime);
    void on_btnLogin_clicked();
    void on_btnLogout_clicked();
    void on_btnPasswordOublie_clicked();
    void on_btnCreerUnCompte_clicked();
    void on_btnOuvrirUnCompte_clicked();
    void on_btnClose_clicked();
    void on_lePassword_returnPressed();
    void on_btnTetiereSimple_clicked();
    void on_lwGrandSSO_itemDoubleClicked(QListWidgetItem *item);
    void on_btnTetiereSimpleRetourPetitSSO_clicked();

protected:
    void mouseMoveEvent(QMouseEvent* event);
    void mousePressEvent(QMouseEvent* event);
    void mouseReleaseEvent(QMouseEvent* event);

private:
    QImage       *greyScale(QImage * origin);
    QPixmap       m_pixmap;    //l'image a utiliser

    bool                           m_moving;
    QPoint                         m_offset;

    Ui::AbulEduSSOLoginV1         *ui;
    MaiaXmlRpcClient              *m_rpcWebadminResetPass;
    bool                           m_logoutInProgress;
    AbulEduNetworkAccessManagerV1 *m_nam;
    int                            m_webadminPassAsked;
    bool                           m_localDebug;
    QString                        m_situation; //pour savoir si on est en train de demander un mot de passe par email etc.
    enumAbulEduSSOLOgin            m_petitOuGrand;

    AbulEduNetworkTests           *m_abuleduNetworkTest;
};

#endif // ABULEDUSSOLOGINV1_H
