/**
  * Classe AbulEduBulleV1
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduPageAccueilV1
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduaccueilbullev1.h"
#include <QDebug>

AbulEduBulleV1::AbulEduBulleV1(QPointF position, QString cheminBulle, QString texte, int offsetVertical, QFont font, float ratio, QGraphicsObject *parent) :
QGraphicsObject(parent)
{
    m_localDebug = 0;
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    m_cheminBulle=cheminBulle;
    m_position = position;
    m_bulle = new QImage(cheminBulle);
    m_texte = texte;
    m_font = font;
    m_ratio = ratio;
    m_offsetVertical = offsetVertical;
    m_couleurFont = QColor(Qt::black);
    setVisible(true);
}

AbulEduBulleV1::~AbulEduBulleV1()
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    delete(m_bulle);
}

void AbulEduBulleV1::abeBulleResize(float ratio)
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_ratio = ratio;
    prepareGeometryChange();
}

void AbulEduBulleV1::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    Q_UNUSED(option);
    Q_UNUSED(widget);

    QFont font = m_font;
    int taillepixel = m_font.pixelSize() *m_ratio;
    font.setPixelSize(taillepixel);
    painter->setFont(font);
    QFontMetrics fontMetric(font);
    int texteSize = fontMetric.width(m_texte);
    QImage image = m_bulle->scaled(texteSize +40 * m_ratio,
                                   m_bulle->height() * m_ratio,
                                   Qt::IgnoreAspectRatio, Qt::SmoothTransformation);


    painter->drawImage(QPoint(m_position.x() * m_ratio, m_position.y() * m_ratio), image);
    painter->setPen(m_couleurFont);
    painter->drawText(QPoint((m_position.x() + 20)* m_ratio,
                             (m_position.y() + m_offsetVertical) * m_ratio),
                      m_texte);
}

QRectF AbulEduBulleV1::boundingRect() const
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    // Pourrait être mieux calculé !!!
    QFont font = m_font;
    int taillepixel = m_font.pixelSize() *m_ratio;
    font.setPixelSize(taillepixel);
    QFontMetrics fontMetric(font);
    int texteSize = fontMetric.width(m_texte);
    return QRectF(QPointF(m_position.x() * m_ratio - 10, m_position.y() * m_ratio - 10),
                  QSizeF(texteSize + 60 * m_ratio,qMax(m_bulle->height()*m_ratio + 30,(float)fontMetric.height() +5)));
}

QPainterPath AbulEduBulleV1::shape() const
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QPainterPath path;
    path.addRect(boundingRect());
    return path;
}

void AbulEduBulleV1::abeBulleSetPosition(QPointF position)
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_position = position;
}

QPointF AbulEduBulleV1::abeBulleGetPosition()
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_position;
}

void AbulEduBulleV1::abeBulleSetImageBulle(QString cheminBulle)
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_bulle = new QImage(cheminBulle);
    m_cheminBulle = cheminBulle;
}

QString AbulEduBulleV1::abeBulleGetImageBulle()
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_cheminBulle;
}

void AbulEduBulleV1::abeBulleSetTexte(QString texte)
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_texte = texte;
}

QString AbulEduBulleV1::abeBulleGetText()
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_texte;
}

void AbulEduBulleV1::abeBulleSetFont(QFont font)
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_font = font;
}

QFont AbulEduBulleV1::abeBulleGetFont()
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_font;
}

void AbulEduBulleV1::abeBulleSetColorPen(QColor color)
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_couleurFont = color;
}

void AbulEduBulleV1::abeBulleSetOffsetV(int offsetV)
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_offsetVertical = offsetV;
}

int AbulEduBulleV1::abeBulleGetOffsetV()
{
    if (m_localDebug) {
        qDebug() << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_offsetVertical;
}
