/**
  * Classe AbulEduZoneV1
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduPageAccueilV1
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduaccueilzonev1.h"
#include <QDebug>
#include <QCursor>

AbulEduZoneV1::AbulEduZoneV1(int numero, QRectF zone, float ratio, QString cheminBulle, QString texteBulle, QFont font, QPoint posBulle, int offsetV, bool actif, QGraphicsItem *parent):
    QObject(0), QGraphicsRectItem(parent)
{
    setAcceptedMouseButtons(Qt::LeftButton);
    setBrush(QColor(Qt::transparent));
    abeZoneDrawRect(false);
    m_numero = numero;
    m_rect = zone;
    m_ratio = ratio;
    m_cheminBulle = cheminBulle;
    m_texteBulle = texteBulle;
    m_font = font;
    m_posBulle = posBulle;
    m_offsetV = offsetV;
    m_actif = actif;
    m_bulle = new AbulEduBulleV1(m_posBulle, m_cheminBulle, m_texteBulle, m_offsetV, m_font);
    m_bulle->setVisible(false);
    if(!m_actif){
        setAcceptHoverEvents(false);
    }
    else
     {
        setAcceptHoverEvents(true);
    }

    abeZoneResizeRect(m_ratio);
    QCursor curseur(Qt::PointingHandCursor);
    if(m_actif)
    {
        QGraphicsItem::setCursor(curseur); // Le curseur devient une petite main, sinon il reste comme il était
    }
}
AbulEduZoneV1::~AbulEduZoneV1()
{

}

AbulEduBulleV1 *AbulEduZoneV1::abeZoneGetBulle()
{
    return m_bulle;
}

QRectF AbulEduZoneV1::abeZoneGetRect()
{
    return m_rect;
}

void AbulEduZoneV1::abeZoneSetActif(bool YesNo)
{
    m_actif = YesNo;
}

bool AbulEduZoneV1::abeZoneIsActif()
{
    return m_actif;
}

void AbulEduZoneV1::abeZoneDrawRect(bool YesNo, QColor color, int thickness, Qt::PenStyle style)
{
    if (YesNo == true)
    {
        setPen(QPen(color,thickness,style));
    }
    else
    {
        setPen(QPen(Qt::NoPen));
    }
    update();
}

void AbulEduZoneV1::abeZoneResizeRect(float ratio)
{
    m_ratio = ratio;
    setRect(m_rect.x() * ratio,
            m_rect.y() * ratio,
            m_rect.width() * ratio,
            m_rect.height() * ratio);
    m_bulle->abeBulleResize(ratio);
}

void AbulEduZoneV1::abeZoneSetRect(QRectF rect)
{
    m_rect = rect;
}

void AbulEduZoneV1::hoverEnterEvent(QGraphicsSceneHoverEvent *)
{
    if(m_actif){
       m_bulle->setVisible(true);
    }
}

void AbulEduZoneV1::hoverLeaveEvent(QGraphicsSceneHoverEvent *)
{
    m_bulle->setVisible(false);
}

int AbulEduZoneV1::abeZoneGetNumero()
{
    return m_numero;
}


