/**
  * Classe
  * @see https://redmine.ryxeo.com/projects/
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  *
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */


#include "abuleduvirtualkeyboardguiv1.h"
#include "ui_abuleduvirtualkeyboardguiv1.h"
#include <QTextEdit>

AbulEduVirtualKeyboardGuiV1::AbulEduVirtualKeyboardGuiV1(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduVirtualKeyboardGuiV1)
{
    m_localDebug       = false;
    m_hasMoved         = false;
    m_viewportHasMoved = false;
    m_inputWidget      = 0;

    m_parent = parent;

    ui->setupUi(this);
    QList<QPushButton *> btnLettres = findChildren<QPushButton *>(QRegExp("btnLettre.*"));
    for(int i = 0; i < btnLettres.size(); i++) {
        if(m_localDebug) {
            qDebug() << "Connect de " << btnLettres.at(i)->objectName();
        }
        connect(btnLettres.at(i), SIGNAL(clicked()), this, SLOT(slotBtnLettreClicked()), Qt::UniqueConnection);
    }
}

AbulEduVirtualKeyboardGuiV1::~AbulEduVirtualKeyboardGuiV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    delete ui;
}

void AbulEduVirtualKeyboardGuiV1::changeClavier()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->btnCHANGE->isChecked()) {
        clavierNUM();
        ui->btnACCENTS->setChecked(false);
        ui->btnSHIFTLeft->setChecked(false);
    }
    else {
        if(ui->btnACCENTS->isChecked()) {
            if(ui->btnSHIFTLeft->isChecked()) {
                clavierAccentsMAJ();
            }
            else {
                clavierAccentsMIN();
            }
        }
        else {
            if(ui->btnSHIFTLeft->isChecked()) {
                clavierNormalMAJ();
            }
            else {
                clavierNormalMIN();
            }
        }
    }
}

void AbulEduVirtualKeyboardGuiV1::clavierNormalMIN()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    // ligne 1
    ui->btnLettreA->setText("a");
    ui->btnLettreZ->setText("z");
    ui->btnLettreE->setText("e");
    ui->btnLettreR->setText("r");
    ui->btnLettreT->setText("t");
    ui->btnLettreY->setText("y");
    ui->btnLettreU->setText("u");
    ui->btnLettreI->setText("i");
    ui->btnLettreO->setText("o");
    ui->btnLettreP->setText("p");

    // ligne 2
    ui->btnLettreQ->setText("q");
    ui->btnLettreS->setText("s");
    ui->btnLettreD->setText("d");
    ui->btnLettreF->setText("f");
    ui->btnLettreG->setText("g");
    ui->btnLettreH->setText("h");
    ui->btnLettreJ->setText("j");
    ui->btnLettreK->setText("k");
    ui->btnLettreL->setText("l");
    ui->btnLettreM->setText("m");

    // ligne 3
    ui->btnLettreW->setText("w");
    ui->btnLettreX->setText("x");
    ui->btnLettreC->setText("c");
    ui->btnLettreV->setText("v");
    ui->btnLettreB->setText("b");
    ui->btnLettreN->setText("n");
}

void AbulEduVirtualKeyboardGuiV1::clavierNormalMAJ()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    // ligne 1
    ui->btnLettreA->setText("A");
    ui->btnLettreZ->setText("Z");
    ui->btnLettreE->setText("E");
    ui->btnLettreR->setText("R");
    ui->btnLettreT->setText("T");
    ui->btnLettreY->setText("Y");
    ui->btnLettreU->setText("U");
    ui->btnLettreI->setText("I");
    ui->btnLettreO->setText("O");
    ui->btnLettreP->setText("P");

    // ligne 2
    ui->btnLettreQ->setText("Q");
    ui->btnLettreS->setText("S");
    ui->btnLettreD->setText("D");
    ui->btnLettreF->setText("F");
    ui->btnLettreG->setText("G");
    ui->btnLettreH->setText("H");
    ui->btnLettreJ->setText("J");
    ui->btnLettreK->setText("K");
    ui->btnLettreL->setText("L");
    ui->btnLettreM->setText("M");

    // ligne 3
    ui->btnLettreW->setText("W");
    ui->btnLettreX->setText("X");
    ui->btnLettreC->setText("C");
    ui->btnLettreV->setText("V");
    ui->btnLettreB->setText("B");
    ui->btnLettreN->setText("N");
}

void AbulEduVirtualKeyboardGuiV1::clavierAccentsMIN()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    // ligne 1
    ui->btnLettreA->setText(trUtf8("à"));
    ui->btnLettreZ->setText(trUtf8("â"));
    ui->btnLettreE->setText(trUtf8("ä"));
    ui->btnLettreR->setText(trUtf8(""));
    ui->btnLettreT->setText(trUtf8(""));
    ui->btnLettreY->setText(trUtf8("ü"));
    ui->btnLettreU->setText(trUtf8("û"));
    ui->btnLettreI->setText(trUtf8("ö"));
    ui->btnLettreO->setText(trUtf8("ô"));
    ui->btnLettreP->setText(trUtf8("œ"));

    // ligne 2
    ui->btnLettreQ->setText(trUtf8("é"));
    ui->btnLettreS->setText(trUtf8("è"));
    ui->btnLettreD->setText(trUtf8("ê"));
    ui->btnLettreF->setText(trUtf8("ë"));
    ui->btnLettreG->setText(trUtf8(""));
    ui->btnLettreH->setText(trUtf8(""));
    ui->btnLettreJ->setText(trUtf8(""));
    ui->btnLettreK->setText(trUtf8(""));
    ui->btnLettreL->setText(trUtf8("î"));
    ui->btnLettreM->setText(trUtf8("ï"));

    // ligne 3
    ui->btnLettreW->setText(trUtf8("ç"));
    ui->btnLettreX->setText(trUtf8(""));
    ui->btnLettreC->setText(trUtf8(""));
    ui->btnLettreV->setText(trUtf8(""));
    ui->btnLettreB->setText(trUtf8(""));
    ui->btnLettreN->setText(trUtf8(""));
}

void AbulEduVirtualKeyboardGuiV1::clavierAccentsMAJ()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    // ligne 1
    ui->btnLettreA->setText(trUtf8("À"));
    ui->btnLettreZ->setText(trUtf8("Â"));
    ui->btnLettreE->setText(trUtf8("Ä"));
    ui->btnLettreR->setText(trUtf8(""));
    ui->btnLettreT->setText(trUtf8(""));
    ui->btnLettreY->setText(trUtf8("Ü"));
    ui->btnLettreU->setText(trUtf8("Û"));
    ui->btnLettreI->setText(trUtf8("Ö"));
    ui->btnLettreO->setText(trUtf8("Ô"));
    ui->btnLettreP->setText(trUtf8("Œ"));

    // ligne 2
    ui->btnLettreQ->setText(trUtf8("É"));
    ui->btnLettreS->setText(trUtf8("È"));
    ui->btnLettreD->setText(trUtf8("Ê"));
    ui->btnLettreF->setText(trUtf8("Ë"));
    ui->btnLettreG->setText(trUtf8(""));
    ui->btnLettreH->setText(trUtf8(""));
    ui->btnLettreJ->setText(trUtf8(""));
    ui->btnLettreK->setText(trUtf8(""));
    ui->btnLettreL->setText(trUtf8("Î"));
    ui->btnLettreM->setText(trUtf8("Ï"));

    // ligne 3
    ui->btnLettreW->setText(trUtf8("Ç"));
    ui->btnLettreX->setText(trUtf8(""));
    ui->btnLettreC->setText(trUtf8(""));
    ui->btnLettreV->setText(trUtf8(""));
    ui->btnLettreB->setText(trUtf8(""));
    ui->btnLettreN->setText(trUtf8(""));
}

void AbulEduVirtualKeyboardGuiV1::clavierNUM()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    // ligne 1
    ui->btnLettreA->setText(trUtf8("1"));
    ui->btnLettreZ->setText(trUtf8("2"));
    ui->btnLettreE->setText(trUtf8("3"));
    ui->btnLettreR->setText(trUtf8("4"));
    ui->btnLettreT->setText(trUtf8("5"));
    ui->btnLettreY->setText(trUtf8("6"));
    ui->btnLettreU->setText(trUtf8("7"));
    ui->btnLettreI->setText(trUtf8("8"));
    ui->btnLettreO->setText(trUtf8("9"));
    ui->btnLettreP->setText(trUtf8("0"));

    // ligne 2
    ui->btnLettreQ->setText(trUtf8("#"));
    ui->btnLettreS->setText(trUtf8("€"));
    ui->btnLettreD->setText(trUtf8("$"));
    ui->btnLettreF->setText(trUtf8("%"));
    ui->btnLettreG->setText(trUtf8("&"));
    ui->btnLettreH->setText(trUtf8("*"));
    ui->btnLettreJ->setText(trUtf8("-"));
    ui->btnLettreK->setText(trUtf8("+"));
    ui->btnLettreL->setText(trUtf8("("));
    ui->btnLettreM->setText(trUtf8(")"));

    // ligne 3
    ui->btnLettreW->setText(trUtf8("<"));
    ui->btnLettreX->setText(trUtf8(">"));
    ui->btnLettreC->setText(trUtf8("="));
    ui->btnLettreV->setText(trUtf8("["));
    ui->btnLettreB->setText(trUtf8("]"));
    ui->btnLettreN->setText(trUtf8(""));
}

void AbulEduVirtualKeyboardGuiV1::slotBtnLettreClicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //    QSound::play(":/abuleduvirtualkeyboardv1/click");
    if(sender()->objectName().startsWith("btnLettre")) {
        QPushButton *btn = static_cast<QPushButton*> (sender());
        if (m_inputWidget) {
            QString charToAdd = btn->text();

            /// Gestion touche Espace
            if(sender()->objectName() == "btnLettreSPACE") {
                charToAdd = " ";
            }
            /// Gestion touche Retour Chariot
            else if(sender()->objectName() == "btnLettreRETURN") {
                charToAdd = "";
            }
            QKeyEvent key(QKeyEvent::KeyPress, 0, Qt::NoModifier, charToAdd, false, 0 );
            QApplication::sendEvent(m_inputWidget, &key);
        }
    }
}

void AbulEduVirtualKeyboardGuiV1::on_btnSHIFTLeft_toggled(bool checked)
{
    Q_UNUSED(checked);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    changeClavier();
}

void AbulEduVirtualKeyboardGuiV1::on_btnBACK_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if (m_inputWidget) {
        QKeyEvent key(QKeyEvent::KeyPress, Qt::Key_Backspace, Qt::NoModifier, 0, false, 0 );
        QApplication::sendEvent(m_inputWidget, &key);
    }
}

void AbulEduVirtualKeyboardGuiV1::on_btnLettreRETURN_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if (m_inputWidget) {
        QKeyEvent key(QKeyEvent::KeyPress, Qt::Key_Enter, Qt::NoModifier, 0, false, 0 );
        QApplication::sendEvent(m_inputWidget, &key);
    }
}

void AbulEduVirtualKeyboardGuiV1::on_btnDroite_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if (m_inputWidget) {
        QKeyEvent key(QKeyEvent::KeyPress, Qt::Key_Right, Qt::NoModifier, 0, false, 0 );
        QApplication::sendEvent(m_inputWidget, &key);
    }

}

void AbulEduVirtualKeyboardGuiV1::on_btnGauche_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if (m_inputWidget) {
        QKeyEvent key(QKeyEvent::KeyPress, Qt::Key_Left, Qt::NoModifier, 0, false, 0 );
        QApplication::sendEvent(m_inputWidget, &key);
    }
}

void AbulEduVirtualKeyboardGuiV1::on_btnClose_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    // Si la fenetre a bougee
    if(m_hasMoved) {
        // restauration de sa position
        m_inputWidget->topLevelWidget()->setGeometry(m_positionOrigine);
        if(m_localDebug)
        {
            qDebug() << "TopLevel m_inputWidget            : " << m_inputWidget->topLevelWidget()->objectName();
            qDebug() << "frameGeometry parent input Widget : " << m_inputWidget->topLevelWidget()->frameGeometry();
            qDebug() << "Geometry parent input Widget      : " << m_inputWidget->topLevelWidget()->geometry();
        }
        m_hasMoved = false;
    }
    // Si le viewport d'un texteEdit a été bougé
    if(m_viewportHasMoved)
    {
        QTextEdit *castTextEdit;
        if(castTextEdit = dynamic_cast<QTextEdit *>(m_inputWidget)) {
            castTextEdit->viewport()->setGeometry(m_positionOrigineViewport);
            castTextEdit->viewport()->setFixedHeight(m_positionOrigineViewport.height());
            m_viewportHasMoved = false;
        }
    }
    // et on cache le clavier
    hide();
}

void AbulEduVirtualKeyboardGuiV1::showEvent(QShowEvent *event)
{
    Q_UNUSED(event)
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "-------------------------------";
        qDebug()  << "Position du clavier virtuel (windows): " << this->window()->geometry();
        qDebug()  << "Position du clavier virtuel (x, y)   : " << this->x() << " " << this->y();
    }

    // Position du clavier en bas
    move(0, (QApplication::desktop()->availableGeometry().height()) - height());

    if (m_inputWidget)
    {
        controlePosCursor(m_inputWidget);
    }
    if (m_inputWidget)
    {
        if(m_localDebug) {
            qDebug() << "Position du LabelEdit (mapToGlobal)    : " << m_inputWidget->mapToGlobal(QPoint(0,0));
            qDebug() << "Position y du Label (mapToGlobal)      : " << m_inputWidget->mapToGlobal(QPoint(0,0)).y();
            qDebug() << "Pos basse du Label (mTG)+height widget : " << m_inputWidget->mapToGlobal(QPoint(0,0)).y() + m_inputWidget->height();
            qDebug() << "Position du LabelEdit                  : " << m_inputWidget->pos();
            qDebug() << "Position du LabelEdit                  : " << m_inputWidget->geometry()  << " "  << m_inputWidget->frameGeometry();
            qDebug() << "Position widget                        : " << QPoint( m_inputWidget->mapToGlobal(QPoint(0,0)).x(), m_inputWidget->mapToGlobal(QPoint(0,0)).y());
            qDebug() << "Position widget + height               : " << QPoint( m_inputWidget->mapToGlobal(QPoint(0,0)).x(), m_inputWidget->mapToGlobal(QPoint(0,0)).y() + m_inputWidget->height());
            qDebug() << "Type de l'objet (pour le caster)       : " << m_inputWidget->objectName();
        }

        /// Il faut le QPoint du bas du widget, sinon probleme pr les widgets pile poil au dessus du clavier
        if (this->geometry().contains(QPoint( m_inputWidget->mapToGlobal(QPoint(0,0)).x(), m_inputWidget->mapToGlobal(QPoint(0,0)).y() + m_inputWidget->height())))
        {
            if(this->height() + m_inputWidget->height() > 600)
            {
                QTextEdit *castTextEdit;
                if(castTextEdit = dynamic_cast<QTextEdit *>(m_inputWidget)) {
                    m_positionOrigineViewport = castTextEdit->viewport()->frameGeometry();
                    int hauteur = m_inputWidget->height() - this->height();
                    castTextEdit->viewport()->setFixedHeight(hauteur);
                    m_viewportHasMoved = true;
                }
            }
            else
            {
                /// Si le label est caché, il faut deplacer la fenetre vers le haut
                /// avant toute chose, sauvegarde de la position de la fenetre parente du labelEdit
                /// remarque = saveGeometry() ne fonctionne pas en restauration (surement probleme de mapToGlobal...)
                m_positionOrigine = m_inputWidget->topLevelWidget()->frameGeometry();
                if(m_localDebug)
                    qDebug() << "Sauvegarde Position Fenetre -------- ok " << m_inputWidget->topLevelWidget()->frameGeometry();
                if(m_localDebug) {
                    qDebug() << "Il est cache";
                    qDebug() << "-------------------------------";
                    qDebug() <<" le y du widget      : " << m_inputWidget->mapToGlobal(QPoint(0,0)).y();
                    qDebug() <<"hauteur du widget    : " << m_inputWidget->height() ;
                }
                int hauteurFenetreplusWidget =  m_inputWidget->mapToGlobal(QPoint(0,0)).y() + m_inputWidget->height();
                int hauteurRestanteAvecClavier = m_inputWidget->topLevelWidget()->height() - this->height();
                m_inputWidget->topLevelWidget()->move(0,(hauteurRestanteAvecClavier - hauteurFenetreplusWidget));
                m_hasMoved = true;
            }
        }
    }
}

void AbulEduVirtualKeyboardGuiV1::on_btnACCENTS_toggled(bool checked)
{
    Q_UNUSED(checked);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    changeClavier();
}

void AbulEduVirtualKeyboardGuiV1::on_btnCHANGE_toggled(bool checked)
{
    Q_UNUSED(checked);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    changeClavier();
}

void AbulEduVirtualKeyboardGuiV1::abeSetInputWidget(QWidget *w)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_inputWidget = w;

    if(dynamic_cast<QTextEdit *>(m_inputWidget))
    {
        connect(m_inputWidget, SIGNAL(cursorPositionChanged()), this, SLOT(testControlePosition()),Qt::UniqueConnection);
    }
    m_inputWidget->setFocus();
}

QWidget *AbulEduVirtualKeyboardGuiV1::abeGetInputWidget()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_inputWidget;
}

void AbulEduVirtualKeyboardGuiV1::testControlePosition()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if (m_inputWidget)
    {
        //        controlePosCursor(m_inputWidget);
    }
    //    qDebug()  << "Le texte a changé";
}

void AbulEduVirtualKeyboardGuiV1::controlePosCursor(QWidget *widget_aControler)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //    QTextEdit *castTextEdit;
    //    castTextEdit = dynamic_cast<QTextEdit *>(widget_aControler);
    //    qDebug() << "Position cursor bas : " << castTextEdit->cursorRect().bottom();
    //    qDebug() << "Position cursor haut: " << castTextEdit->cursorRect().top();
    //    qDebug() << "Position clavier    : " << this->frameGeometry().top();
    //    qDebug() << "Position cursor2    : " << castTextEdit->cursorRect();
    //    qDebug() << "Position clavierf   : " <<this->frameGeometry();

    //    qDebug() << "pos cursor : " << QRect(castTextEdit->mapToGlobal(castTextEdit->cursorRect().bottomLeft()), castTextEdit->cursorRect().size());

    //    if (this->geometry().contains(QRect(castTextEdit->mapToGlobal(castTextEdit->cursorRect().bottomLeft()), castTextEdit->cursorRect().size())))
    //    {
    //        qDebug() << "Le curseur est cache !!!!!";
    //        // Faire les calculs :)
    //        qDebug() << "Geo du castEdit : " << castTextEdit->geometry();
    //        qDebug() << "View du castEdit : " << castTextEdit->viewport()->geometry();
    //        //        qDebug() << castTextEdit->
    //    }
}
