/**
  * (le terrier d'AbulEdu)
  *
  * @warning aucun traitement d'erreur n'est pour l'instant implémenté
  * @see https://redmine.ryxeo.com/projects/leterrier-aller
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUWIDGETASSISTANTETAPESV1_H
#define ABULEDUWIDGETASSISTANTETAPESV1_H

#include <QWidget>
#include <QCloseEvent>
#include <QList>
#include <QPair>
#include <QMap>
#include <QPushButton>
#include <QFont>
#include <QDebug>
#include "abuleduflatboutonv1.h"
#include <QSignalMapper>

#ifndef DEBUG_ASSISTANTETAPESV1
#define DEBUG_ASSISTANTETAPESV1 0
#endif

namespace Ui {
class AbulEduWidgetAssistantEtapesV1;
}

class AbulEduWidgetAssistantEtapesV1 : public QWidget
{
    Q_OBJECT
    
public:
    /** Constructeur
      * Permet de créer un widget visualisant le chemin dans un assistant Editeur
      * @param listeEtapes Une QList de QPair<QString, QString> first est le nom de l'étape, second est le nom abrégé de l'étape
      */
    explicit AbulEduWidgetAssistantEtapesV1(QList<QPair<QString, QString> > listeEtapes,
                                            int smallFontSize = 12,
                                            int mediumFontSize= 16,
                                            int largeFontSize = 20,
                                            QWidget *parent   = 0 );
    ~AbulEduWidgetAssistantEtapesV1();


    /** Spécifie les trois tailles des polices */
    void abeWidgetAssistantSetFontSize(int smallFontSize, int mediumFontSize, int largeFontSize);

    /** Crée les boutons des étapes
      * @param listeEtapes Une QList de QPair<QString, QString> first est le nom de l'étape, second est le nom abrégé de l'étape
      */
    void abeWidgetAssistantCreateEtapes(QList<QPair<QString, QString> > listeEtapes);

    /** Active le bouton suivant ou pas */
    void abeWidgetAssistantEnableSuivant(bool enabled);

    /** Active le bouton retour ou pas */
    void abeWidgetAssistantEnablePrecedent(bool enabled);

    /** Active la navigation via les boutons */
    void abeWidgetAssistantEnableClick(bool yesNo = true);

    /** Retourne la liste des boutons étapes
      * @return QList des pointeurs vers les boutons
      */
    QList<AbulEduFlatBoutonV1 *> abeWidgetAssistantGetButtons();

    /** Retourne un pointeur vers le bouton numéro n
      * @param numéro du bouton
      * @return pointeur vers le bouton
      */
    AbulEduFlatBoutonV1 *abeWidgetAssistantGetButton(int i);

    /** Retourne un pointeur vers le bouton suivant
      * @return pointeur vers le bouton
      */
    AbulEduFlatBoutonV1 *abeWidgetAssistantGetButtonNext();

    /** Retourne un pointeur vers le bouton précédent
      * @return pointeur vers le bouton
      */
    AbulEduFlatBoutonV1 *abeWidgetAssistantGetButtonPrevious();

    /** Retourne un pointeur vers le bouton quitter
      * @return pointeur vers le bouton
      */
    AbulEduFlatBoutonV1 *abeWidgetAssistantGetButtonQuit();

private:
    Ui::AbulEduWidgetAssistantEtapesV1 *ui;

    QMap<int, QPair<AbulEduFlatBoutonV1 *, QPair<QString, QString> > > m_MapBoutonsEtapes;
    int m_etapeCourante;
    int m_nbEtapes;

    QFont m_smallFont;
    QFont m_mediumFont;
    QFont m_largeFont;
    bool m_localDebug;
    bool m_buttonsActivated;
    QSignalMapper *m_signalMapper;
private slots:

    void boutonEtapeClicked(int numBouton);

    void on_pbRetour_clicked();

    void on_pbSuivant_clicked();

    void on_pbQuitter_clicked();

public slots:
    /** Spécifie une étape comme étape courante
      * @param etapeCourante (0 pour Accueil)
      */
    void abeWidgetAssistantSetEtapeCourante(int etapeCourante);
signals:
    /** Signal émis lors du changement d'étape
      * @param retourne le numéro de la nouvelle étape (0 pour Accueil)
      */
    void signalEtapeHasChanged(int numEtape);

    /** Signal émis lors de l'appui sur pbSuivant */
    void signalEtapeSuivanteRequested();

    /** Signal émis lors de l'appui sur pbRetour */
    void signalEtapePrecedenteRequested();

    /** Signal émis lors de l'appui sur pbQuitter */
    void signalQuitterRequested();
};

#endif // ABULEDUWIDGETASSISTANTETAPESV1_H
